function [StructConstLad, AdjRepLad, Roots] = lietyperank2structconst( Control, type, rank)
%LIECARTAN2STRUCTURECONSTANTS    Lie structure constants from Cartan matrix
%
%  [StructConstLad, AdjRepLad, Roots] = lietyperank2structconst( Control, type, rank)
%
%    AdjRepLad : adj representation (ladder coordinate system)
%
%    Control.Visual 
%    Control.Verbose 
%    Control.Testing 
%    Control.PlotRootSystem
%    Control.SaveStructConst 
%    Control.OutDir = '/home/gbeyerle/data/math/lie/';
%
%  References:
%
%    R. J. Riebeek; Computations in Association Schemes, 
%      PhD dissertation, 1998
%    R.B Howlett, L.J Rylands, D.E Taylor;
%      Matrix Generators for Exceptional Groups of Lie Type
%      Journal of Symbolic Computation, Volume 31, Issue 4, April 2001, 
%      Pages 429–445, doi:10.1006/jsco.2000.0431
%    R. W. Carter, Simple groups of Lie type, John Wiley, 1972 
%

% (c) G. Beyerle 2010-2012
%
% This program comes with ABSOLUTELY NO WARRANTY. This is free software,
% and you are welcome to redistribute it under certain conditions;

%
%  define:
%
%  H_{alpha}   elements of Cartan subalgebra
%  E_{+beta}   +beta : positive roots             
%  E_{-beta}   -beta : negative roots            
%
%  zero generators (elements of cartan subalgebra commute)
%
%    [H_{beta},H_{alpha}]  = 0                         
%
%    [H_{beta},E_{+alpha}] = 2 (beta,alpha) / (beta,beta) E_{+alpha}
%
%    [H_{alpha},E_{gamma}] = 2 (alpha,gamma) / (alpha,alpha) E_{gamma}
%
%  H_{alpha} are _not_ linearly independent
%  E_{gamma} are linearly independent
%
%  [E_{+beta},E_{-beta}] = H_{beta}
%
%  if beta+gamma not a root:
%
%    [E_{beta},E_{gamma}]  = 0                     
%
%  if beta+gamma is a root:
%
%    [E_{beta},E_{gamma}]  = N_{beta,gamma} E_{beta+gamma}  
%                          = +/-(p+1) E_{beta+gamma}  
%
%  H_{beta} = sum k * H_k       H_k : basis of Cartan subalgebra
%           = sum_i=1^rank k_i (alpha_i,alpha_i) / (beta,beta) H_i


%
%  - sort roots first
%  - optimize findROOTS
%

%   disp( [upper( mfilename) ': FIXME - convert to integer arithmetic!'])
%   disp( [upper( mfilename) ': FIXME - normalize ROOTSRS to unity for simple roots!'])

  if 0,
    NCoeff = lietyperank2structconst( [], 'g', 2);
  end
  
  if 0,
    NCoeff = lietyperank2structconst( [], 'e', 6);
  end

  if 0,
    c.SaveStructConst=1;
    for r=2:30,lietyperank2structconst( c, 'd', r); end
  end
  
  clear global
  
%
%  check if Lie algebra of chosen type and rank actually exists.
%
  local_validateinput( type, rank);

%
%  initialize control parameter
%
  Control = local_setcontrol( Control);

  dynkin = zeros( 1, rank);
  [roots, ~, ~, ~, cartan, metric, ...
    invcartan] = lietyperank2rootsweights( Control, type, rank, dynkin);
    
%   roots
%   cartan
%   metric
%   keyboard
  
%
%  remove zero roots
%  (Lie algebra A1 (rank 1) is special case)
%
  if rank > 1,
    roots(:,all(roots==0)) = [];
  else
    roots(:,roots==0) = [];
  end
    
%
%  lietyperank2rootsweights() returns integer variables
%  
  roots     = double( roots);
  cartan    = double( cartan);
  invcartan = double( invcartan);
  metric    = double( metric);

%   cartan
% %  cartan = cartan(end:-1:1,end:-1:1);
%   cartan = cartan';
%   cartan
%   keyboard
  
%
%  there exists a well-defined order of roots
%  a) r < s <=> HEIGHTOFROOTS(r) < HEIGHTOFROOTS(s) 
%  b) if more than one root share the same height, then
%     sort roots lexicographically
%
  RootOrder = local_initROOTSORDER( roots, invcartan);
    
  global ROOTS
  roots = roots(:,RootOrder);
  ROOTS = roots;

  global INVCARTAN
  INVCARTAN = invcartan;

%
%  ROOTSRS : roots expressed in the simple root coordinate system
%
  global ROOTSRS
  ROOTSRS = invcartan * roots;
%  ROOTSRS = double( invcartan) * double( roots);

  if 0,
    NofRoots = size( ROOTS, 2);

    %  round( 2 * inv( roots(end:-1:1,NofRoots/2+(rank:-1:1))))
    Tmp       = eye(rank);
    TmpRevCol = Tmp(end:-1:1,:);
    Tmp = ( TmpRevCol * inv( roots(:,NofRoots/2+(1:rank))));
    MinTmp = abs(min(Tmp(Tmp~=0)));
    Tmpinvcartan = round( Tmp / MinTmp)

    invcartan
    Tmpinvcartan

    TmpROOTSRS = Tmpinvcartan * roots;

    ROOTSRS(:,NofRoots/2+(1:rank))
    TmpROOTSRS(:,NofRoots/2+(1:rank))
%    priv_assert( ROOTSRS == TmpROOTSRS)
    keyboard

%     Tmp * cartan
%     invcartan * cartan
%     keyboard
  end
  
  
% 
% 
%   %  round( 2 * inv( roots(end:-1:1,NofRoots/2+(rank:-1:1))))
%   invcartan
%   cartan
%   Tmp * cartan
%   invcartan * cartan
%   keyboard


%   ROOTSRS
%   keyboard

  global HEIGHTOFROOTS
%   heightOfRts = sum( rootsSRS, 1);


%   Tmp = ROOTSRS(:,end/2+rank:-1:end/2+1);
%   priv_assert( all( all( Tmp(1,1)*eye( rank) == Tmp)))  
%   if Tmp(1,1) ~= 1
%     disp( [upper( mfilename) ...
%       ': normalization of rootSRS     = ' num2str( Tmp(1,1))])
%     NrmROOTSRS = ROOTSRS / Tmp(1,1);
%   else
%     NrmROOTSRS = ROOTSRS / Tmp(1,1);
%   end

%  NrmHEIGHTOFROOTS = sum( NrmROOTSRS, 1);

  global ROOTLEN

  NofRoots = size( ROOTS, 2);
  ROOTLEN = zeros( 1, NofRoots);
  for i = 1:NofRoots
    ROOTLEN(i) = ROOTS(:,i)' * metric * ROOTS(:,i);
  end

  ROOTLEN = ROOTLEN / min( ROOTLEN);
  
  HEIGHTOFROOTS    = sum( ROOTSRS, 1);

%   HEIGHTOFROOTS    = sum( ROOTSRS, 1) .* ROOTLEN;
% 
%   sum( ROOTSRS, 1)
%   HEIGHTOFROOTS
%   ROOTLEN
%   keyboard

%   priv_clfig(99)
%   plot( NrmHEIGHTOFROOTS, '.')
% %   priv_clfig(98)
% %   plot( sum( ROOTSRS, 1) ./ sum( NrmROOTSRS, 1), '.')
%   keyboard
  
% flag indicates positive or negative root
% SIGNOFROOTS(r) = +1 -> root r is positive root
% SIGNOFROOTS(r) = -1 -> root r is negative root
  global SIGNOFROOTS
  SIGNOFROOTS = sign( HEIGHTOFROOTS);
  
  local_initROOTLEN( metric);
   
%
%  SUMOFROOTS(r,s) is the root k=i+j with
%   [e_{r},e_{s}] = N_(rs) e_{r+s}
%
%  SUMOFROOTS(r,s) = 0 can mean two things
%
%  a) r+s is not a root, 
%  b) r+s is a zero root (a root corresponding to an 
%     element within the Cartan subalgebra)
%
  local_initSUMOFROOTS();

%
%  root j = ROOTFLIPSIGN(i) is the 
%  a) negative root j corresponding to root i, if i is positive or
%  b) positive root j corresponding to root i, if i is negative.
%
  local_initROOTFLIPSIGN();

%  get simple roots from Cartan matrix
%
%  simpleroot1     = double(cartan(:,1))'
%  simpleroot2     = double(cartan(:,2))'
    
%   
%   smplRootsIdx = local_findsimpleroots( roots, cartan);
%   
% %   inv( roots(smplrootsidx,:)') * roots(1,:)'
% %   roots(smplrootsidx,:)' \ roots(1,:)'  
% %   inv( roots(smplrootsidx,:)')
%     
%   [Tmp, SortIdx] = sort( ROOTLEN(smplRootsIdx));
%   
% %
% %  sort fundamental/simple roots according to length
% %
%   smplRootsIdx = smplRootsIdx(SortIdx);
% 
% %   IsSimpleRoot = zeros( 1, nofRoots);
% %   IsSimpleRoot(smplrootsidx) = 1;

%  express roots in coordinates w.r.t. simple roots
%  rootsSRS : roots in terms of simple roots
%
%   global HEIGHTOFROOTS
%   [HEIGHTOFROOTS, rootsSRS] = local_initHEIGHTOFROOTS( invcartan, cartan);
   
%   emptyCells = cellfun( @isempty, strfind( Control.Testing, 'sumofroot'));
%   if any( ~emptyCells)
%     local_initSUMOFROOTSX();
% %     SUMOFROOTS
% %     SUMOFROOTSX
% %     keyboard
%   end
  
%
%  do some consistency check on the roots
%
  local_checkroots( cartan, metric);
%   disp('transposed cartan matrix!')
%   local_checkroots( cartan');
%   keyboard
  
  
%   ROOTS(:,ROOTSORDER)
%   -ROOTS(end:-1:1,ROOTSORDER(end:-1:1))
%   keyboard
  
%  priv_assert( all( ROOTS(:,ROOTSORDER) + ROOTS(:,ROOTSORDER(end:-1:1)) == 0))
  priv_assert( all( ROOTS + ROOTS(:,end:-1:1) == 0))
%  priv_assert( all( ROOTS(:,ROOTSORDER) + ROOTS(:,ROOTSORDER(end:-1:1)) == 0))
  
%   roots'
%   ROOTSORDER
%   keyboard

% --- plot root system ---  
  if Control.PlotRootSystem 
    local_plotrootsystem2d3d( invcartan, cartan);
  end
% --- plot root system ---  

%
%  PCOEFF(i,j) is the greatest integer such that j - k * i is a root 
%
  local_initPCOEFF();
      
%
%  find extraspecial pairs of roots
%
%  r and s are special if r, s, r+s are positive roots and r < s
%
%  the special pair (r,s) is called extraspecial if for all other 
%  special pairs (r1,s1) with r1+s1=r+s we have r1 > r
%
  local_initEXTRASPECIALPAIR( Control);
      
  local_initISEXTRASPECIALPAIR();
     
%
%  r and s are special if r, s, r+s are positive root and r < s
%
  local_initISSPECIALPAIR( Control);

%
%  check correctness of special pairs 
%
  local_checkspecialpairs( )

%
%  determine structure constants [E_a,E_b] = N_ab E_{a+b}
%

%
%  Source:
%    R. J. Riebeek; Computations in Association Schemes, 
%      PhD dissertation, 1998
%
  if 0,
    NCoeffRiebeek = local_assembleNCoeffRiebeek();
  else
    NCoeffRiebeek = [];
  end
  
  if 1,

%  Source:
%    R.B Howlett, L.J Rylands, D.E Taylor;
%      Matrix Generators for Exceptional Groups of Lie Type
%      Journal of Symbolic Computation, Volume 31, Issue 4, April 2001, 
%      Pages 429–445, doi:10.1006/jsco.2000.0431
%
    NCoeffHowlett = local_assembleNCoeffHowlett();
        
%    priv_assert( all( NCoeffHowlett == NCoeffHowlettOld))
%     if ~all( all( NCoeffHowlett == NCoeffRiebeek))
%       [i,j] = find( NCoeffHowlett ~= NCoeffRiebeek);
%       [i,j]
%       SUMOFROOTS
%       NCoeffRiebeek
%       NCoeffHowlett
%       keyboard
%     end
  else
    NCoeffHowlett = [];
  end
  
%  NCoeff = NCoeffRiebeek;
  NCoeff = NCoeffHowlett;

  if ~isempty( NCoeffHowlett) && ~isempty( NCoeffRiebeek)
    priv_assert( NCoeffHowlett == NCoeffRiebeek)
  end
  
%   NCoeffRiebeek
%   NCoeffHowlett
%   keyboard
  
  global SUMOFROOTSX
  emptyCells = cellfun( @isempty, strfind( Control.Testing, 'ncoeff'));
  if any( ~emptyCells)
    if isempty( SUMOFROOTSX)
      local_initSUMOFROOTSX();
    end
    local_checkNCoeff( NCoeff);
  end
  
  if 1
    NCoeffVal = sort( unique( NCoeff))';
    [a,b] = hist( NCoeff(:), min(NCoeffVal):max(NCoeffVal));
    if ~isempty( a)
      disp( [upper( mfilename) ...
        ': values in struct. constants  = ' sprintf( '%8d', b)])
      disp( [upper( mfilename) ...
        ': abundancy of these values    = ' sprintf( '%8d', a)])
    end
%    keyboard
  end
  
%
%  now we (almost) all entries to fill the structure constant matrix
%
%  Note:  AdjRepLad = permute( StructConstLad, [3 2 1]); 
%
%         AdjRepLad(:,:,k)  : adj. representation of k-th generator

  [StructConstLad, AdjRepLad] = priv_structconstandadjrepr( metric, NCoeff);
  
  AdjRepRot = priv_transformadjrep( rank, AdjRepLad, type);

  StructConstRot = permute( AdjRepRot, [3 2 1]); 

%  priv_testadjointrepresentation( rank, StructConstRot, AdjRepRot);
  
  if 0,
  %  N = size( StructConstRot, 1);
    for i=1:NofRoots
      for j=1:NofRoots
        Idx = find( StructConstRot(i,j,1:NofRoots));
        if length( Idx) > 1
          [i,j]
          StructConstRot(i,j,1:NofRoots)
          keyboard
        end
      end
    end
  %  StructConstRot
    keyboard
  end
  
  Tmp = StructConstRot(1:NofRoots,1:NofRoots,1:NofRoots);
  NCoeffRot = sum( double( Tmp ~= 0), 3);
%   whos  
%   keyboard
  
  if 0,
    TestOutFile = plotfile( ['43129-' upper( type) num2str( rank) '-a']);
    disp( TestOutFile)
    save( TestOutFile, 'StructConstLad', 'AdjRepLad')
    keyboard
  end
  
  if 0,
    N = size( StructConstLad, 1);
    KillingForm = zeros( N, N); 
    for i = 1:N,
      for j = 1:N,
  %      diag( AdjRep(:,:,i) * AdjRep(:,:,j))'
         KillingForm(i,j) = trace( AdjRepLad(:,:,i) * AdjRepLad(:,:,j));
       end
    end
    
    KillingForm(end-rank+1:end,end-rank+1:end)
    fs    = 18;
    FigNo = 99;
    nofRoots = size( ROOTS, 2);
    Tmp = KillingForm;
  %  Tmp(Tmp==0) = NaN;

    [xyRootNo2D, zzCMIdx, colMap, ...
      UniqTmp] = local_preparecolordata( Tmp(1:nofRoots,1:nofRoots), nofRoots);

    FigNo = 99;
    priv_clfig(FigNo)
    image( xyRootNo2D, xyRootNo2D, zzCMIdx);
    set( gca, 'ydir', 'normal');
    colormap( colMap);
    set(gca,'fontsize',fs)
    hold on
    plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
    plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
    set(gca,'tickdir','out')
    set(gca,'dataaspect',[1 1 1])
    xlabel( 'root number')
    ylabel( 'root number')
  %  title( ['Structure constants of Lie algebra ' upper(type) num2str( rank) ])
    title( ['Killing form  (' upper(type) num2str( rank) ')'])
    local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));
    keyboard
  end
  
  
  
                                  
%   if exist( OutFile, 'file')
% %    OutFile
% %    keyboard
%     load( OutFile, 'NCoeffSrt', 'RootsSrt')
%     NCoeffSrt
%     NCoeff
%     [i, j] = find( NCoeffSrt ~= NCoeff);
%     [i, j]
%     length( find( NCoeffSrt ~= NCoeff))
%     priv_assert( all( NCoeffSrt == NCoeff))
%     priv_assert( all( RootsSrt == roots))
%     keyboard
%   end
  
%  StructCoeff = NCoeff;
  Cartan      = cartan;
  InvCartan   = INVCARTAN;
  Metric      = metric;
  Roots       = roots;

  if Control.SaveStructConst,
    OutFile = [Control.OutDir 'liestructconst-' upper( type) num2str( rank) '.mat'];
    disp( [upper( mfilename) ': ' OutFile])
    makedirectory( OutFile)
    save( OutFile, 'StructConstLad', 'Roots', 'Cartan', 'InvCartan', 'Metric')
  end  
  
%  return
  
  emptyCells = cellfun( @isempty, strfind( Control.Testing, 'wybourne'));
  if any( ~emptyCells),
    if isempty( SUMOFROOTSX)
      local_initSUMOFROOTSX();
    end
    tic
    priv_testwybournetheorem81( metric, NCoeff);
%    priv_validatecartertheorem82( metric, NCoeffRiebeek);
    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing priv_testwybournetheorem81() >' ...
      num2str( ElapsedTime) ' sec<'])
  end
  
  emptyCells = cellfun( @isempty, strfind( Control.Testing, 'jacobi'));
  if any( ~emptyCells),
    if isempty( SUMOFROOTSX)
      local_initSUMOFROOTSX();
    end
    tic
    priv_testjacobiidentity( metric, NCoeff);
    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing priv_testjacobiidentity() >' ...
      num2str( ElapsedTime) ' sec<'])
  end
  
  emptyCells = cellfun( @isempty, strfind( Control.Testing, 'adjrep'));
  if any( ~emptyCells),

%    priv_testadjointrepresentation( rank, permute( AdjRep, [3 2 1]), permute( StructConstLad, [3 2 1]));
%    priv_testadjointrepresentation( rank, permute( AdjRep, [2 1 3]), permute( StructConstLad, [2 1 3]));
%    keyboard
        
    tic
    priv_testadjointrepresentation( rank, StructConstLad, AdjRepLad);
    priv_testadjointrepresentation( rank, StructConstRot, AdjRepRot);
    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing priv_testadjointrepresentation() >' ...
      num2str( ElapsedTime) ' sec<'])

  end
  
  if 0,
    if isempty( SUMOFROOTSX)
      local_initSUMOFROOTSX();
    end
    priv_check4jacobiidentity( metric, NCoeff);
  end
  
%   NCoeffRiebeekSrt
%   -NCoeffRiebeekSrt(end:-1:1,end:-1:1)
  
%  [i,j] = find( rem( NCoeffRiebeekSrt, 1) ~= 0);
  
%  priv_assert( NCoeffRiebeekSrt == -NCoeffRiebeekSrt(end:-1:1,end:-1:1))
  priv_assert( NCoeff == -NCoeff(end:-1:1,end:-1:1))
  priv_assert( all( rem( NCoeff, 1) == 0))
  priv_assert( NCoeff == -NCoeff')
  
  if ~Control.Visual
    return
  end
  
  
%   Sel = find( NCoeffRiebeek);
%   priv_assert( abs( NCoeffRiebeek(Sel)) == PCOEFF(Sel)+1)
%  [abs( NCoeffRiebeek(Sel)), PCOEFF(Sel)+1]
  
%   tic
%   for i = 1:nofRoots,
%     for j = i+1:nofRoots,
%       TmpNCoeff(i,j) = local_structconst_sav( i, j);
%     end
%   end
%   toc

%  keyboard
  

%   tic
%   for i = 1:nofRoots,
%     for j = 1:nofRoots,
%       TmpNCoeff(i,j) = local_structconst( i, j);
%     end
%   end
%   toc
% 
%   priv_assert( TmpNCoeff == -TmpNCoeff')
%   priv_assert( TmpNCoeff == NCoeffRiebeek)

%  priv_assert( NCoeffRiebeek == TmpNCoeff)

%   Ratio1 = length( find( NCoeffRiebeek + NCoeffRiebeek(end:-1:1,end:-1:1))) / length( find( NCoeffRiebeek));
%   Ratio2 = length( find( abs( sign( NCoeffRiebeek)) - ...
%                          abs( sign( NCoeffRiebeek(end:-1:1,end:-1:1))))) / length( find( NCoeffRiebeek));
%   
%   Ratio1
%   Ratio2
  



% --- plot result ---  
  fs    = 18;
  FigNo = 0;
%  Tmp = NCoeffRiebeek;
  nofRoots = size( ROOTS, 2);

  Tmp = NCoeff;
%  Tmp(Tmp==0) = NaN;
  
  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( Tmp, nofRoots);

  FigNo = FigNo+1;
  priv_clfig(FigNo)
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
  colormap( colMap);
  set(gca,'fontsize',fs)
  hold on
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set(gca,'tickdir','out')
  set(gca,'dataaspect',[1 1 1])
  xlabel( 'root number')
  ylabel( 'root number')
%  title( ['Structure constants of Lie algebra ' upper(type) num2str( rank) ])
  title( ['Structure constants  (' upper(type) num2str( rank) ')'])
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));


  fs    = 18;
  FigNo = FigNo+1;
  nofRoots = size( ROOTS, 2);

  Tmp = NCoeffRot;
  Tmp(Tmp==0) = NaN;
  
  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( Tmp, nofRoots);

  FigNo = FigNo+1;
  priv_clfig(FigNo)
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
  colormap( colMap);
  set(gca,'fontsize',fs)
  hold on
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set(gca,'tickdir','out')
  set(gca,'dataaspect',[1 1 1])
  xlabel( 'root number')
  ylabel( 'root number')
%  title( ['Structure constants of Lie algebra ' upper(type) num2str( rank) ])
  title( ['# non-zero strct. const. (rot.) (' upper(type) num2str( rank) ')'])
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));
   
%  keyboard
  
  if 0
    FigNo = FigNo+1;
    priv_clfig(FigNo)
    image( xyRootNo2D, xyRootNo2D, zzCMIdx);
    set( gca, 'ydir', 'normal');
    colormap( colMap);
%     h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%     set( h, 'edgecolor', 'none')
%     colormap( cmSC);
    set( gca, 'xlim', [1-0.5,nofRoots/2+0.5])
    set( gca, 'ylim', [1-0.5,nofRoots/2+0.5])
    set(gca,'dataaspect',[1 1 1])
    set( gca, 'visible', 'off')
    grafeps([ '/home/gbeyerle/work/visuallietheory-' lower(type) num2str( rank) '-2.eps'])
    grafjpeg([ '/home/gbeyerle/work/visuallietheory-' lower(type) num2str( rank) '-2.jpeg'])

    FigNo = FigNo+1;
    priv_clfig(FigNo)
    image( xyRootNo2D, xyRootNo2D, zzCMIdx);
    set( gca, 'ydir', 'normal');
    colormap( colMap);
%     h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%     set( h, 'edgecolor', 'none')
%     colormap( cmSC);
    set( gca, 'xlim', [-nofRoots/2-0.5, -0.5])
    set( gca, 'ylim', [0.5, nofRoots/2+0.5])
    set(gca,'dataaspect',[1 1 1])
    set( gca, 'visible', 'off')
    grafeps([ '/home/gbeyerle/work/visuallietheory-' lower(type) num2str( rank) '-3.eps'])
    grafjpeg([ '/home/gbeyerle/work/visuallietheory-' lower(type) num2str( rank) '-3.jpeg'])
  end
  
%
%  plot only left-right symmetric structure constants
%
  TmpNCoeff = NCoeff;
  Idx = find( TmpNCoeff == 0 | TmpNCoeff(:,end:-1:1) == 0);
  TmpNCoeff(Idx) = 0;
  TmpNCoeff(TmpNCoeff==0) = NaN;
%   Tmp = local_addNaNcolumnsandrows( TmpNCoeff, nofRoots);
%   xyRootNo2D = (-nofRoots/2:nofRoots/2+1)-0.5;

  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( TmpNCoeff, nofRoots);

%  zzVal = unique( Tmp);
%  zzVal(isnan(zzVal)) = [];
  
  FigNo = FigNo+1;
  priv_clfig(FigNo)
%  h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%  set( h, 'edgecolor', 'none')
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
%   cm = local_creatcolormap( unique( NCoeffSrt(:)));
%   colormap( cm)
%  colormap( hsv)
%  cm = local_creatcolormap( zzVal);
  colormap( colMap);
%  xcolorbar('vert',(min(Tmp(:)):max(Tmp(:))),0);
  set(gca,'fontsize',fs)
  hold on
%   plot( xyRootNo2D, zeros( size( xyRootNo2D)), 'k-')
%   plot( zeros( size( xyRootNo2D)), xyRootNo2D, 'k-')
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set(gca,'tickdir','out')
  set(gca,'dataaspect',[1 1 1])
  xlabel( 'root number')
  ylabel( 'root number')
  title( ['L/R symmetry of struct. const.  (' upper(type) num2str( rank) ')' ])
%  local_addcoloredboxcaption( zzVal, cmSC);
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));
  

%
%  plot only left-right symmetric structure constants
%
  TmpNCoeff = zeros( size( NCoeff));
  Idx = find( NCoeff == 0 & NCoeff(:,end:-1:1) == 0);
  TmpNCoeff(Idx) = 1;
%  TmpNCoeff(TmpNCoeff==0) = NaN;
%   Tmp = local_addNaNcolumnsandrows( TmpNCoeff, nofRoots);
%   xyRootNo2D = (-nofRoots/2:nofRoots/2+1)-0.5;

  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( TmpNCoeff, nofRoots);

%  zzVal = unique( Tmp);
%  zzVal(isnan(zzVal)) = [];
  
  FigNo = FigNo+1;
  priv_clfig(FigNo)
%  h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%  set( h, 'edgecolor', 'none')
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
%   cm = local_creatcolormap( unique( NCoeffSrt(:)));
%   colormap( cm)
%  colormap( hsv)
%  cm = local_creatcolormap( zzVal);
  colormap( colMap);
%  xcolorbar('vert',(min(Tmp(:)):max(Tmp(:))),0);
  set(gca,'fontsize',fs)
  hold on
%   plot( xyRootNo2D, zeros( size( xyRootNo2D)), 'k-')
%   plot( zeros( size( xyRootNo2D)), xyRootNo2D, 'k-')
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set(gca,'tickdir','out')
  set(gca,'dataaspect',[1 1 1])
  xlabel( 'root number')
  ylabel( 'root number')
  title( ['L/R symmetry of struct. const.  (' upper(type) num2str( rank) ')' ])
%  local_addcoloredboxcaption( zzVal, cmSC);
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));
  
  
  
  global PCOEFF

  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( PCOEFF, nofRoots);

%  xyRootNo = (-nofRoots/2:nofRoots/2+1)-0.5;
  FigNo = FigNo+1;
  priv_clfig(FigNo)
%   h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%   cmPC = local_creatcolormap( zzVal);
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
  colormap( colMap)
%  colormap( 'default')
%  colormap( hsv)
%  colormap( hot)
%  set( h, 'edgecolor', 'none')
  hold on
%   plot( xyRootNo2D, zeros( size( xyRootNo2D)), 'k-')
%   plot( zeros( size( xyRootNo2D)), xyRootNo2D, 'k-')
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set( gca, 'tickdir','out')
  set( gca, 'dataaspect',[1 1 1])
  set( gca, 'fontsize',fs)
  xlabel( 'root number')
  ylabel( 'root number')
  title( ['P coefficient  (' upper(type) num2str( rank) ')' ])
 % local_addcoloredboxcaption( zzVal, cmPC);
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));


  Tmp = local_addNaNcolumnsandrows( NCoeff, nofRoots);
  Tmp(isnan(Tmp)) = 0;
  xyRootNo = (-nofRoots/2:nofRoots/2);

  FigNo = FigNo+1;
  priv_clfig(FigNo)
  yy = sum( sign( abs( Tmp)));
  yy((end+1)/2) = NaN;
  plot( xyRootNo, sum( sign( abs( Tmp))), 'b.');
  set( gca, 'xlim', [-nofRoots/2-1, nofRoots/2+1])
  set( gca, 'ylim', [min(yy)-1, max(yy)+1])
  set( gca, 'fontsize',fs)
  xlabel( 'root number')
  ylabel( '# nonzero coeff.')
  
  Tmp = HEIGHTOFROOTS;
  yy = [Tmp(1:nofRoots/2), NaN, Tmp((1:nofRoots/2)+nofRoots/2)];
  
  FigNo = FigNo+1;
  priv_clfig(FigNo)
  plot( xyRootNo, yy, 'b.');
  set( gca, 'xlim', [-nofRoots/2-1, nofRoots/2+1])
  set( gca, 'fontsize',fs)
  xlabel( 'root number')
  ylabel( 'height of root')
  title( ['Lie algebra ' upper(type) num2str( rank) ])

  global SUMOFROOTS
  TmpSUMOFROOTS = SUMOFROOTS;
  TmpSUMOFROOTS(TmpSUMOFROOTS == 0) = NaN;
%  Tmp = local_addNaNcolumnsandrows( SUMOFROOTS, nofRoots);

  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( TmpSUMOFROOTS, nofRoots);
  
  Len = size( colMap, 1);
  HLen = (Len-1)/2;
  Idx1 = find( zzCMIdx == 1);
  Idx2 = find( zzCMIdx > 1 & zzCMIdx <= HLen+1);
  zzCMIdx(Idx2) = zzCMIdx(Idx2) - 1;
  zzCMIdx(Idx1) = HLen+1;
  SavCM  = colMap(1,:);
  colMap = [colMap(2:HLen+1,:); SavCM; colMap(HLen+2:end,:)];
  
%   Tmp(isnan(Tmp)) = 0;
%   zzVal = unique( Tmp);

  FigNo = FigNo+1;
  priv_clfig(FigNo)
%  cmSoR = local_creatcolormap( zzVal);
%   h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%   set( h, 'edgecolor', 'none')
%  set( h, 'edgecolor', 'none')
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
%  colormap( jet);
  colormap( colMap);
  hold on
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set( gca, 'tickdir','out')
  set( gca, 'dataaspect',[1 1 1])
  set( gca, 'fontsize',fs)
  xlabel( 'root number')
  ylabel( 'root number')
  title( ['Sum of two roots  (' upper(type) num2str( rank) ')' ])
  cb = colorbar;
% play dirty tricks with colorbar y-axis tick marks  
  set( cb, 'YTickLabelMode', 'manual');
  YTLStr = get( cb, 'YTickLabel');
  YTLStr = num2str( round( str2num( YTLStr) - nofRoots/2 - 1));
  set( cb, 'YTickLabel', YTLStr);
  
  Tmp = SUMOFROOTS;
  Tmp(Tmp==0) = [];
  Idx = find( ~( NCoeff == 0 | NCoeff(:,end:-1:1) == 0));
  Tmp2 = SUMOFROOTS;
  Tmp2(Idx) = 0;
  Tmp2(Tmp2==0) = [];
    
  FigNo = FigNo+1;
  priv_clfig(FigNo)
  [a, b] = hist(Tmp,1:nofRoots);
  yy = [a(1:nofRoots/2) NaN a(nofRoots/2+1:nofRoots)];
  plot( xyRootNo, yy, 'rp')
   hold on
   if ~isempty( Tmp2)
     [a, b] = hist(Tmp2,1:nofRoots);
     yy2 = [a(1:nofRoots/2) NaN a(nofRoots/2+1:nofRoots)];
     plot( xyRootNo, yy2, 'bo')
   end
  set( gca, 'xlim', [-nofRoots/2-1, nofRoots/2+1])
  set( gca, 'ylim', [min(yy)-1, max(yy)+1])
  set( gca, 'fontsize',fs)
  xlabel( 'root number')
  ylabel( 'root number occurrence')
  title( ['Lie algebra ' upper(type) num2str( rank) ])

  global EXTRASPECIALPAIR
  global ISSPECIALPAIR

  FigNo = FigNo+1;
  priv_clfig(FigNo)
  Tmp2 = EXTRASPECIALPAIR;
  Tmp1 = zeros(nofRoots,2);
  k=1;
  for i = 1:nofRoots,
    for j = 1:nofRoots,
      if ISSPECIALPAIR(i,j)
        Tmp1(k,1) = i;
        Tmp1(k,2) = j;
        k=k+1;
      end
    end
  end
  Idx = find( Tmp1(:,1) == 0 & Tmp1(:,2) == 0);
  Tmp1(Idx,:)=[];
  Idx = find( Tmp2(:,1) == 0 & Tmp2(:,2) == 0);
  Tmp2(Idx,:)=[];
  plot( Tmp1(:,1)-nofRoots/2, Tmp1(:,2)-nofRoots/2, 'b.');
  hold on
  plot( Tmp2(:,1)-nofRoots/2, Tmp2(:,2)-nofRoots/2, 'r.');
  set( gca, 'tickdir','out')
  set( gca, 'dataaspect',[1 1 1])
  set( gca, 'fontsize',fs)
  set( gca, 'xlim', [0, nofRoots/2+1])
  set( gca, 'ylim', [0, nofRoots/2+1])
  xlabel( '(extra)special pair 1')
  ylabel( '(extra)special pair 2')
  title( ['Lie algebra ' upper(type) num2str( rank) ])
    
  RootTimesRoot = zeros( nofRoots, nofRoots);
  for i = 1:nofRoots
    for j = 1:nofRoots
      RootTimesRoot(i,j) = ROOTS(:,i)' * metric * ROOTS(:,j);
    end
  end
  Idx = find( RootTimesRoot == 0);
    disp( [upper( mfilename) ': perpendicular root pairs     = ' ...
      num2str( length( Idx)) ' out of ' num2str( nofRoots.^2)  ...
      ' ( ' num2str( length( Idx) / nofRoots.^2) ' )'])
  
  [xyRootNo2D, zzCMIdx, colMap, ...
    UniqTmp] = local_preparecolordata( RootTimesRoot, nofRoots);
  
  FigNo = FigNo+1;
  priv_clfig(FigNo)
%  h = pcolor( xyRootNo2D, xyRootNo2D, Tmp);
%  set( h, 'edgecolor', 'none')
  image( xyRootNo2D, xyRootNo2D, zzCMIdx);
  set( gca, 'ydir', 'normal');
  colormap( colMap);
%  xcolorbar('vert',(min(Tmp(:)):max(Tmp(:))),0);
  set(gca,'fontsize',fs)
  hold on
  plot( [-(nofRoots+1)/2, (nofRoots+1)/2], [0, 0], 'k-')
  plot( [0, 0], [-(nofRoots+1)/2, (nofRoots+1)/2], 'k-')
  set(gca,'tickdir','out')
  set(gca,'dataaspect',[1 1 1])
  xlabel( 'root number')
  ylabel( 'root number')
  title( ['Root angles  (' upper(type) num2str( rank) ')' ])
  local_addcoloredboxcaption( UniqTmp, colMap(2:end,:));

%  keyboard
  drawnow

end

% -------------------------------------------------------------------------
%
%  prepare data to plot using image()
%
function [xyRootNo2D, zzCMIdx, colMap, ...
  UniqTmp] = local_preparecolordata( RootTimesRoot, nofRoots)

%  keyboard

  Tmp        = local_addNaNcolumnsandrows( RootTimesRoot, nofRoots);
  xyRootNo2D = (-nofRoots/2:nofRoots/2);
  zzCMIdx     = ones( size( Tmp)); % 1 -> NaN -> grey
  NotNaNIdx  = ~isnan( Tmp);
  [UniqTmp, ~, TmpIdx] = unique( Tmp(NotNaNIdx));

  % plot NaN as grey  
  
%  cm = local_createcolormap( length( UniqTmp));
  cm = local_createcolormap( UniqTmp);
  if isempty( cm),
    cm = jet( length( UniqTmp));
  end
  
%  colMap = [[0.8,0.8,0.8]; cm];
  colMap = [[0.9,0.9,0.9]; cm];
  
  zzCMIdx(NotNaNIdx)   = TmpIdx+1; % first entry in colormap is grey 

  % plot zero as white  
  Idx = find( UniqTmp == 0);
  if ~isempty( Idx),
    colMap(Idx+1,:) = [1 1 1];
  end
  
end

% -------------------------------------------------------------------------
%
%  add figure caption
%
function local_addcoloredboxcaption( zzVal, cm)

%   zzVal
%   whos
%   keyboard

  nofzzVal = length( zzVal);

%  yAxOfs = nofzzVal/2; 
  
  priv_assert( nofzzVal == size( cm, 1))

%  axes( 'position', [.85  0.1  .05  0.8])
  axes( 'position', [.85  -0.1  .05  1])
%  axes( 'position', [.85  0.5*yAxOfs  .05  0.8])

  plot(NaN)
  hold on
  set(gca,'visible','off')
  set(gca,'dataaspect',[1 1 1])
  set(gca,'xlim',[-0.1 1.1])
  set(gca,'ylim',[-3 nofzzVal+4])
  dx   = 1;
  dy   = 1;
  xOfs = 0;
  yOfs = -dy/2;
  cmLen = size( cm, 1);
  
%  priv_mlassert( )
  
  jj=1;
%  zzVal = [zzVal(1:(cmLen-1)/2); 0; zzVal((cmLen-1)/2+1:end)];
  for j=1:cmLen
%     if j == (cmLen+1)/2
%       continue
%     end
    patch( [0,dx,dx,0,0]+xOfs, [0,0,dy,dy,0]+yOfs, cm(j,:))
    text( xOfs+dx*1.7, yOfs+dy*0.45, num2str( zzVal(jj)), ...
      'fontsize', 16, 'verticalalign', 'middle', 'horizontalalign', 'center')
    jj=jj+1;
    yOfs = yOfs + 1.5*dy;
  end
end

% % -------------------------------------------------------------------------
% %
% %
% %
% %
% function Cof = local_getcoeff2( i, j, NCoeff)
%         
%   global ROOTFLIPSIGN
%   global SIGNOFROOTS
% 
%   if ( i == ROOTFLIPSIGN( j))
% % pos/neg root pair!    
%     Cof  = 1 * SIGNOFROOTS(i);
% %    Cof  = 1;
%   else
%     Cof  = NCoeff(i,j);
%   end
%   
% end

% -------------------------------------------------------------------------
%
%  add NaN in center (horizontally and vertically)
%
%
function Tmp = local_addNaNcolumnsandrows( Tmp, nofRoots)

%  Tmp(Tmp == 0) = NaN;
% add row/col with NaN for pcolor  
%  Tmp = [[Tmp, ones(nofRoots,1)*NaN]; ones(1,nofRoots+1)*NaN];
  N = nofRoots/2;

%   Tmp = [Tmp(:,1:N), ones(nofRoots,1)*NaN, Tmp(:,N+1:2*N), ones(nofRoots,1)*NaN];
%   Tmp = [Tmp(1:N,:); ones(1,nofRoots+2)*NaN; Tmp(N+1:2*N,:); ones(1,nofRoots+2)*NaN];
  Tmp = [Tmp(:,1:N), ones(nofRoots,1)*NaN, Tmp(:,N+1:2*N)];
  Tmp = [Tmp(1:N,:); ones(1,nofRoots+1)*NaN; Tmp(N+1:2*N,:)];

end

% -------------------------------------------------------------------------
%
%
%
function Control = local_setcontrol( Control)

  if ~isfield( Control, 'Visual')
    Control.Visual = 0;
  end
  
  if ~isfield( Control, 'Verbose')
    Control.Verbose = 0;
  end
  
  if ~isfield( Control, 'Testing')
    Control.Testing = {};
  else
    if ~iscell( Control.Testing)
      Control.Testing = {Control.Testing};
      disp( [upper( mfilename) ': Control.Testing converted to single cell.'])
    end
  end

  if ~isfield( Control, 'PlotRootSystem')
    Control.PlotRootSystem = 0;
  end
  
  if ~isfield( Control, 'SaveStructConst')
    Control.SaveStructConst = 0;
  end
  
  if ~isfield( Control, 'OutDir') && exist( 'hostname', 'file')
    switch lower( hostname)
      case {'kg77a','kistegebe'}
        Control.OutDir = '/home/gbeyerle/data/math/lie/';
      case {'champ96'}
        Control.OutDir = '/home/kg/gbeyerle/data/math/lie/';
      otherwise
    end
  end

  
  if Control.SaveStructConst 
    disp( [upper( mfilename) ': Control.Visual = 0'])
    Control.Visual = 0;
  end

  CtrlTestingAllowed = {'adjrep','wybourne','sumofroot','ncoeff','jacobi'};
  Len = length( Control.Testing);
  for i = 1:Len
    emptyCells = cellfun( @isempty, strfind( CtrlTestingAllowed, Control.Testing{i}));
    if all( emptyCells)
      error( [upper( mfilename) ': unknown option Control.Testing = >' Control.Testing{i} '< ' ...
        '(allowed options: ''adjrep'',''wybourne'',''sumofroot'',''ncoeff'',''jacobi'')'])
    end
  end
  
  
end

% -------------------------------------------------------------------------
%
%  r and s are special if r, s, r+s are positive roots and r < s
%
function local_initISSPECIALPAIR( Control)

  global ROOTS
%  global ROOTSORDER
  global SIGNOFROOTS
  global SUMOFROOTS
  global ISSPECIALPAIR
  
  nofRoots     = size( ROOTS, 2);
  rank         = size( ROOTS, 1);

%  ISSPECIALPAIR = zeros( nofRoots, nofRoots, 'int8');
  ISSPECIALPAIR = zeros( nofRoots, nofRoots);

  if Control.Verbose
    disp([upper( mfilename) ': special pairs are ...'])
  end
  
  for i = 1:nofRoots  
    for j = 1:nofRoots  
      k = SUMOFROOTS(i,j);
      if k == 0,
        continue
      end
      if SIGNOFROOTS(i) < 0 || SIGNOFROOTS(j) < 0 
        continue
      end
%      if ROOTSORDER(i) < ROOTSORDER(j)
      if i < j
        ISSPECIALPAIR(i,j) = 1;
% -------------        
        if Control.Verbose
          Str = '';
          for ll = 1:rank  
            Str = [Str, sprintf( '%d', ROOTS(ll,i))];
            if ll < rank
              Str = [Str, ','];
            end
          end
          Str = [Str, ') + ('];
          for ll = 1:rank  
            Str = [Str, sprintf(  '%d', ROOTS(ll,j))];
            if ll < rank
              Str = [Str, ','];
            end
          end
          Str = [Str, ') = ('];
          for ll = 1:rank  
            Str = [Str, sprintf(  '%d', ROOTS(ll,k))];
            if ll < rank
              Str = [Str, ','];
            end
          end
          disp(sprintf( 'i =%3d  j =%3d  i+j =%3d   (%s)', i, j, k, Str))
        end
% -------------        
      end
    end
  end
  
  return
  
  ROOTS
%  ROOTSORDER
  SIGNOFROOTS
  IsSpecialPair
  keyboard
  
end

% -------------------------------------------------------------------------
%
%  check if pair (r,s) is extra special
%
%
function local_initISEXTRASPECIALPAIR()
  
  global ROOTS
  global EXTRASPECIALPAIR
  global ISEXTRASPECIALPAIR
  
  nofRoots     = size( ROOTS, 2);
  rank         = size( ROOTS, 1);

%  ISEXTRASPECIALPAIR = zeros( nofRoots, nofRoots, 'int8');
  ISEXTRASPECIALPAIR = zeros( nofRoots, nofRoots);
  
  for i = 1:nofRoots  
    Tmp = EXTRASPECIALPAIR(i,:);
    if all( Tmp == 0)
      continue
    end
    ISEXTRASPECIALPAIR(Tmp(1),Tmp(2)) = 1;
  end
  
  priv_assert( length( find( any( EXTRASPECIALPAIR'))) == size( ROOTS, 2)/2 - rank)
  
end
  
% -------------------------------------------------------------------------
%
%  find extraspecial pairs of roots
%
%  r and s are special if r, s, r+s are positive roots and r < s
%
%  the special pair (r,s) is called extraspecial if for all other 
%  special pairs (r1,s1) with r1+s1=r+s we have r1 > r
%
%  the pair (ExtraSpecialPair(k,1),ExtraSpecialPair(k,2)) is the
%  extraspecial pair belonging to root k   
%
function local_initEXTRASPECIALPAIR( Control)
  
  global ROOTS
%  global ROOTSORDER
  global SIGNOFROOTS
  global SUMOFROOTS
  global EXTRASPECIALPAIR
  
  nofRoots     = size( ROOTS, 2);
  rank         = size( ROOTS, 1);

%  EXTRASPECIALPAIR = zeros( nofRoots, 2, 'int32');
  EXTRASPECIALPAIR = zeros( nofRoots, 2);


  for i = 1:nofRoots  
%     if ~IsSimpleRoot(i)
%       continue
%     end
    for j = 1:nofRoots,
      if SIGNOFROOTS(i) < 0 || SIGNOFROOTS(j) < 0
% extraspecial pairs are positive roots        
        continue
      end
      k = SUMOFROOTS(i,j);
      if k == 0,
        continue;
      end
      if EXTRASPECIALPAIR(k,1) == 0  
% pair (i,j) not yet encountered         
        EXTRASPECIALPAIR(k,1) = i;
        EXTRASPECIALPAIR(k,2) = j;
%      elseif ROOTSORDER( ExtraSpecialPair(k,1)) > ROOTSORDER(i) 
      elseif EXTRASPECIALPAIR(k,1) > i 
% pair (i,j) already yet encountered, this one is smaller         
        EXTRASPECIALPAIR(k,1) = i;
        EXTRASPECIALPAIR(k,2) = j;
      end
    end
  end

  if Control.Verbose
    disp([upper( mfilename) ': extraspecial pairs are ...'])
  end
  
  for n = 1:nofRoots  
% -------------        
% extraspecial pairs are positive roots        
    if all( EXTRASPECIALPAIR(n,:) == 0)
      continue
    end
    i = EXTRASPECIALPAIR(n,1);
    j = EXTRASPECIALPAIR(n,2);
    k = SUMOFROOTS(i,j);
    if Control.Verbose
      Str = '';
      for ll = 1:rank  
        Str = [Str, sprintf( '%d', ROOTS(ll,i))];
        if ll < rank
          Str = [Str, ','];
        end
      end
      Str = [Str, ') + ('];
      for ll = 1:rank  
        Str = [Str, sprintf(  '%d', ROOTS(ll,j))];
        if ll < rank
          Str = [Str, ','];
        end
      end
      Str = [Str, ') = ('];
      for ll = 1:rank  
        Str = [Str, sprintf(  '%d', ROOTS(ll,k))];
        if ll < rank
          Str = [Str, ','];
        end
      end
      disp(sprintf( 'i =%3d  j =%3d  i+j =%3d   (%s)', i, j, k, Str))
    end
% -------------        
  end
  
    
end

% -------------------------------------------------------------------------
%
%  determine ordering of roots
%
%  there is a well-defined order of roots
%  a) r < s <=> HEIGHTOFROOTS(r) < HEIGHTOFROOTS(s) 
%  b) if more than one root share the same height, then
%    sort roots lexigraphically
%
function RootOrder = local_initROOTSORDER( roots, invcartan)

%   global ROOTS
%   global INVCARTAN
%  global SIGNOFROOTS
  
%  nofRoots  = size( ROOTS, 2);
  nofRoots  = size( roots, 2);
%  rank      = size( ROOTS, 1);

  RootOrder = zeros(1,nofRoots) * NaN;
  
%  height is sum of coefficients when this root is 
%  expressed as sum of simple roots  

%  TmpInvRoot = INVCARTAN * ROOTS;
%  TmpInvRoot = double( invcartan) * double( roots);
  TmpInvRoot = invcartan * roots;

%   [Tmp2, SortRIdx] = sortrows( Tmp');
%   Tmp
%   SortRIdx'
%   keyboard  
  
  Height = sum( TmpInvRoot, 1);
  
  HeightUniq = sort( unique( Height));
  
  k = 1;
  for h = HeightUniq,
    Sel = find( Height == h);
    SelLen = length( Sel);
    if SelLen == 1,
      RootOrder(Sel) = k;
      k = k + 1;
    else
% if two or more roots share the same height, sort lexicographically
      Sel = sort(Sel);
      [Tmp2, SortRIdx] = sortrows( TmpInvRoot(:,Sel)');
%      SortRIdx = SortRIdx(end:-1:1);
%      SortRIdx
%      keyboard
%       if SIGNOFROOTS(Sel(1)) < 0
%         SortRIdx = SortRIdx(end:-1:1);
%       end
%       TmpInvRoot(:,Sel)
%       TmpInvRoot(:,Sel(SortRIdx))
%       for s = SortRIdx'
%         RootOrder(Sel(s)) = k;
%         disp( sprintf( 'k=%d, Sel(s) = %d(%d) RootOrder(Sel(s))=%d', k, Sel(s), s, RootOrder(Sel(s))))
%         k = k+1;
%       end
      RootOrder(k+(0:SelLen-1)) = Sel(SortRIdx);
      k = k + SelLen;
    end
  end

  return
  
  TmpInvRoot
  RootOrder
  Height
  TmpInvRoot(:,RootOrder)
%  -TmpInvRoot(:,RootOrder(end:-1:1))

  keyboard

end

% % -------------------------------------------------------------------------
% %
% %  sign of root (positive or negative root?)
% %
% function SignOfRoots = local_initSIGNOFROOTS()
% 
%   global ROOTS
%   global ROOTSRS
%   
%   nofRoots    = size( ROOTS, 2);
% 
%   SignOfRoots = zeros( 1, nofRoots);
% 
% %
% %  find positive roots
% %
%   Tmp = zeros( 1, nofRoots);
%   for i = 1:nofRoots,
% %    PosIdx = find( rootsSRS(:,i) > 0, 1);
%     PosIdx = find( ROOTSRS(:,i) > 0, 1);
%     if isempty( PosIdx)
%       continue
%     end
% %    NegIdx = find( rootsSRS(:,i) < 0, 1);
%     NegIdx = find( ROOTSRS(:,i) < 0, 1);
%     if isempty( NegIdx) || ~isempty( NegIdx) && PosIdx < NegIdx,
%       Tmp(i) = 1;
%     end
%   end
% %  posRootsIdx = find( Tmp); 
%   SignOfRoots(Tmp ~= 0) =  1;
% 
%   ROOTSRS
%   keyboard
%   
% % %
% % %  find negative roots
% % %
% %   Tmp = zeros( 1, nofRoots);
% %   for i = 1:nofRoots,
% % %    NegIdx = find( rootsSRS(:,i) < 0, 1);
% %     NegIdx = find( ROOTSRS(:,i) < 0, 1);
% %     if isempty( NegIdx)
% %       continue
% %     end
% % %    PosIdx = find( rootsSRS(:,i) > 0, 1);
% %     PosIdx = find( ROOTSRS(:,i) > 0, 1);
% %     if isempty( PosIdx) || ~isempty( PosIdx) && NegIdx < PosIdx,
% %       Tmp(i) = 1;
% %     end
% %   end
% % %  negRootsIdx = find( Tmp);
% %   SignOfRoots(Tmp ~= 0) =  -1;
% 
% end
  
% -------------------------------------------------------------------------
%
%  PCoeff(r,s) is the largest integer 
%  such that
%
%    s - PCoeff(r,s) * r  
%
%  is a root
%
function local_initPCOEFF()
  
  global ROOTS
  global SUMOFROOTS
  global ROOTFLIPSIGN
  global PCOEFF
  
  nofRoots = size( ROOTS, 2);

%  PCOEFF = zeros( nofRoots, nofRoots, 'int8');
  PCOEFF = zeros( nofRoots, nofRoots);

  for i = 1:nofRoots
    for j = 1:nofRoots
      
      if SUMOFROOTS(i,j) == 0
%         disp( ['root(' num2str(i) ',:) = [' num2str( roots(i,:)) ...
%             '], root(' num2str(j) ',:) = [' num2str( roots(j,:)) ']']);
        continue;
      end

%
%  is alpha - k * beta a root?
%
      iTmp = i;
      jNeg = ROOTFLIPSIGN(j);
      
%
%  there are at most 4 roots in one chain
%
      for k = 1:3,

%        iTmp -> roots(i,:) - k * roots(j,:);
% k = 1: testroot = alpha - 1 * beta
% k = 2: testroot = alpha - 2 * beta =  alpha - beta - beta
% k = 3: testroot = alpha - 3 * beta =  alpha - beta - beta - beta
        
        iTmp = SUMOFROOTS(iTmp,jNeg);
             
        if iTmp == 0,
          break
        end
        
        PCOEFF(i,j) = k;

      end
    end
  end
  
  if max( abs( PCOEFF(:))) ~= 0
    PCoeffVal = sort( unique( PCOEFF))';
    disp( [upper( mfilename) ': nonzero entries in matrix P  = ' num2str( PCoeffVal(2:end))])
%    keyboard
  end

end

% -------------------------------------------------------------------------
%
%  fill array with root indices translating 
%  from positive root to negative root or vice versa
%
function local_initROOTFLIPSIGN()

  global ROOTS
  global ROOTFLIPSIGN

  nofRoots     = size( ROOTS, 2);

  ROOTFLIPSIGN = zeros( 1, nofRoots);

  for i = 1:nofRoots,
    rootIdx = local_findroot( -ROOTS(:,i));
    if rootIdx > 0,
      ROOTFLIPSIGN(i) = rootIdx;
    end
  end
  
end  
  
% % -------------------------------------------------------------------------
% %
% %  fill array with root indices translating 
% %  from positive root to negative root or vice versa
% %
% function RootFlipSignX = local_initROOTFLIPSIGNX()
% 
%   global ROOTS
% 
%   nofRoots     = size( ROOTS, 2);
% 
%   RootFlipSignX = zeros( 1, nofRoots+2);
% 
%   for i = 1:nofRoots,
%     rootIdx = local_findroot( -ROOTS(:,i));
%     if rootIdx > 0,
%       RootFlipSignX(i) = rootIdx;
%     end
%   end
%   RootFlipSignX(nofRoots+1) = nofRoots+1;
%   RootFlipSignX(nofRoots+2) = nofRoots+2;
%   
% end

% -------------------------------------------------------------------------
%
% sum of two roots; e.g. alpha_i + alpha_j = alpha_k
%
%    k = SUMOFROOTS(i,j)
%
% if k == 0, then alpha_i + alpha_j is not a root
%
function local_initSUMOFROOTS()

  global ROOTS
  global SUMOFROOTS

  nofRoots      = size( ROOTS, 2);

%   tic
%   SumOfRoots = zeros( nofRoots, nofRoots);
%   for i = 1:nofRoots,
%     for j = 1:nofRoots,
%       rootIdx = local_findroot( ROOTS(:,i) + ROOTS(:,j));
%       if rootIdx > 0,
%         SumOfRoots(i,j) = rootIdx;
%       end
%     end
%   end
%   toc

%
%  addition of two roots 
%  a) sum is commutative (i+j = j+i)
%  b) i + i is never a root
%
%  SUMOFROOTS = zeros( nofRoots, nofRoots, 'int32');
  SUMOFROOTS = zeros( nofRoots, nofRoots);
  
  tic
  for i = 1:nofRoots,
    for j = i+1:nofRoots,
      rootIdx = local_findroot( ROOTS(:,i) + ROOTS(:,j));
      if rootIdx > 0,
        SUMOFROOTS(i,j) = rootIdx;
      end
    end
  end
  ElapsedTime = toc;
  disp( [upper( mfilename) ': timing local_initSUMOFROOTS() >' num2str( ElapsedTime) ' sec<'])
  
% commutativity: copy upper triangle to lower triangle  
  SUMOFROOTS = SUMOFROOTS + SUMOFROOTS';

%   TmpSumOfRoots = SumOfRoots;
%   TmpSumOfRoots = TmpSumOfRoots + TmpSumOfRoots';
%   for i = 1:nofRoots,
%     for j = 1:i-1,
%       SumOfRoots(i,j) = SumOfRoots(j,i);
%     end
%   end
%   
%   priv_assert( TmpSumOfRoots == SumOfRoots)
%   keyboard
  
end
  
% -------------------------------------------------------------------------
%
% sum of two roots; e.g. alpha_i + alpha_j = alpha_k
%
%    k = SUMOFROOTS(i,j)
%
% if k == NR+1,...,2*NR, then alpha_i + alpha_j is zero root corresp. to H_k
% if k == 2*nofRoots+1, then alpha_i + alpha_j is not a zero root
%
%
function local_initSUMOFROOTSX()

  global ROOTS
  global SIGNOFROOTS
  global SUMOFROOTSX

  %  global CARTANX

  NR      = size( ROOTS, 2);

%  SUMOFROOTSX = zeros( 2*NR+1, 2*NR+1, 'int32');
  SUMOFROOTSX = zeros( 2*NR+1, 2*NR+1);

  for i = 1:NR,
    for j = 1:NR,
      if all( ROOTS(:,i) + ROOTS(:,j) == 0)
% root i is antipodal to j (i+j) is zero root 
        if SIGNOFROOTS(i) > 0
% [E_{i},E_{j}] = [E_{i},E_{-i}] = H_{i} 
          SUMOFROOTSX(i,j) = NR+i;
        else
% [E_{i},E_{j}] = -[E_{j},E_{i}] = -[E_{j},E_{-j}] = -H_{j} 
          SUMOFROOTSX(i,j) = NR+j;
        end
        continue
      end
      rootIdx = local_findroot( ROOTS(:,i) + ROOTS(:,j));
      if rootIdx > 0,
        SUMOFROOTSX(i,j) = rootIdx;
      else
% i + j are not a root        
        SUMOFROOTSX(i,j) = (2*NR+1);
      end
    end
  end
  
%  [h_r,e_s] = A_{rs} e_s  
%   SumOfRootsX(NR+(1:NR),1:NR) = repmat( (1:NR)+NR, NR, 1);
%   SumOfRootsX(1:NR,NR+(1:NR)) = repmat( (1:NR)'+NR, 1, NR);
  SUMOFROOTSX(NR+(1:NR),1:NR) = repmat( (1:NR), NR, 1);
  SUMOFROOTSX(1:NR,NR+(1:NR)) = repmat( (1:NR)', 1, NR);
% %  [h_r,h_s] = 0  
%   SumOfRootsX(NR+1,NR+1)   = NR+2;
%  [0,e_s] = 0 = [e_s,0]    [0,h_s] = 0 = [h_s,0]   
  SUMOFROOTSX(:,2*NR+1)    = 2*NR+1;
  SUMOFROOTSX(2*NR+1,:)    = 2*NR+1;
  
end

% -------------------------------------------------------------------------
% 
%   find simple roots in list of pos & neg roots
% 
function smplrootsidx = local_findsimpleroots( roots, cartan)

  nofRoots      = size( roots, 2);
  numsmplroots  = size( cartan, 2);

  smplrootsidx = ones( 1, numsmplroots) * NaN;
  
%  keyboard
  
  for i = 1:numsmplroots

%    delta = sum( abs( cartan(:,i*ones(1,nofRoots))' - roots'), 2);
    delta = sum( abs( cartan(:,i*ones(1,nofRoots)) - roots), 1);
    
%     delta
%     keyboard
    
    MinIdx = find( delta == 0);
    
    if length( MinIdx) < 1
      cartan
      roots
      error( [upper( mfilename) ': cannot find simple root [' ...
        num2str( cartan(:,i)') '] in list of roots.'])
    elseif length( MinIdx) > 1
      cartan
      roots
      error( [upper( mfilename) ': found simple root [' ...
        num2str( cartan(:,i)') '] more than once in list of roots.'])
    else
      smplrootsidx(i) = MinIdx;
    end
    
  end
    
  priv_assert( length( smplrootsidx) == length( unique( smplrootsidx)))
  
end

% -------------------------------------------------------------------------
% 
% 
% 
% 
% 
% 
function local_arrowplot( xy, col)

  idx = find( all( xy == 0));
  xy(:,idx) = [];

  if isempty( xy)
    return
  end
  
%  xy = xy';
    
%   arrow( zeros(size(xy)), xy, 36, 'BaseAngle', 60, ...
%     'FaceColor', col, 'EdgeColor', col, 'length', 20, 'width', 2) 

  for i = 1:size(xy,2)
    arrow( zeros(size(xy(:,i))), xy(:,i), 36, 'BaseAngle', 60, ...
      'FaceColor', col, 'EdgeColor', col, 'length', 20, 'width', 2) 
%    keyboard
  end
  
%  compass( xy(1,:), xy(2,:))
  
end

% ------------------------------------------------------------------------
%
%  check if Lie algebra of chosen type and rank actually exists.
%
function local_validateinput( type, rank)

  Fail = 0;

  switch upper( type)
    case 'A',      
      if rank < 1 
        Fail = 1;
      end
    case 'B',      
      if rank < 2  
        Fail = 1;
      end
    case 'C',      
      if rank < 3  
        Fail = 1;
      end
    case 'D',      
      if rank < 4  
        Fail = 1;
      end
    case 'E',      
      if rank < 6 || rank > 8
        Fail = 1;
      end
    case 'F',      
      if rank ~= 4 
        Fail = 1;
      end
    case 'G',      
      if rank ~= 2
        Fail = 1;
      end
    otherwise
      Fail = 1;
  end
  
  if Fail,
    error( [ upper( mfilename) ': Lie algebra >' ...
      upper( type) num2str( rank) '< does not exist.'])      
  end
  
end

% -------------------------------------------------------------------------
%
%  plot 2-d or 3-d root systems
%
function local_plotrootsystem( rank, rootsSRS, smplrootsidx)
  
  if rank == 2
  
    xMax = max( rootsSRS(1,:)) + 0.5;
    yMax = max( rootsSRS(2,:)) + 0.5;

    priv_clfig(1)
    plot(NaN)
    set( gca, 'dataaspect',[1 1 1])
    set( gca, 'xlim', [-xMax, xMax])
    set( gca, 'ylim', [-yMax, yMax])
    hold on
    Tmp = rootsSRS;
    Tmp(:,smplrootsidx) = [];
    local_arrowplot( Tmp, 'b');
    local_arrowplot( rootsSRS(:,smplrootsidx), 'r');
    keyboard

  elseif rank == 3

    xMax = max( rootsSRS(1,:)) + 0.5;
    yMax = max( rootsSRS(2,:)) + 0.5;
    zMax = max( rootsSRS(3,:)) + 0.5;

    priv_clfig(1)
    plot3(NaN,NaN,NaN)
    set( gca, 'dataaspect',[1 1 1])
    set( gca, 'xlim', [-xMax, xMax])
    set( gca, 'ylim', [-yMax, yMax])
    set( gca, 'zlim', [-zMax, zMax])
    hold on
    Tmp = rootsSRS;
    Tmp(:,smplrootsidx) = [];
    local_arrowplot( Tmp, 'b');
    local_arrowplot( rootsSRS(:,smplrootsidx), 'r');
    keyboard

  end
  

end

% % -------------------------------------------------------------------------
% %
% %
% %
% %
% %
% function CartanX = local_initCARTANX( metric)
% 
%   global ROOTS
%               
%   numrts = size( ROOTS, 2);
% 
%   CartanX = zeros( numrts, numrts);
%   
%   for i = 1:numrts,
%     for j = 1:numrts,
%       CartanX(i,j) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
%                         (ROOTS(:,i)' * metric * ROOTS(:,i));
% %       CartanX(j,i) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
% %                         (ROOTS(:,i)' * metric * ROOTS(:,i));
%     end
%   end
% 
% end


% % -------------------------------------------------------------------------
% %
% %  find index into list of roots ROOTS corresponding to root rt
% %  
% %    http://stackoverflow.com/questions/6209904/matlab-find-given-row-in-a-matrix
% %
% function rootIdx = local_findroot2( rt)
% 
%   global ROOTS
%   global INVCARTAN
%   global HEIGHTOFROOTS
% 
%   rtSRS = INVCARTAN * rt;
% 
%   Sel = find( sum( rtSRS) == HEIGHTOFROOTS);
%   if isempty( Sel)
%     rootIdx = 0;
%     return;
%   end
% 
% %   delta = sum( abs( rt(:,ones(1,length(Sel))) - ROOTS(:,Sel)), 1);
% %   [MinVal, MinIdx] = min( delta);
% %   if MinVal == 0,
% %     rootIdx = Sel(MinIdx);
% %   end
%    
%   [~,TmpIdx] = ismember( rt', ROOTS(:,Sel)', 'rows');  
% 
%   if TmpIdx
%     rootIdx = Sel(TmpIdx);
%   else
%     rootIdx = 0;
%   end  
%   
% end

% -------------------------------------------------------------------------
%
%  find index into list of roots ROOTS corresponding to root rt
%
function rootIdx = local_findroot( rt)

  global ROOTS
  global INVCARTAN
  global HEIGHTOFROOTS
  
%  persistent JUSTONCE   
% %
% %  1) determine sign of ROOTS and rt
% %  2) search only within subset of corresponding sign
% %
%   if isempty( JUSTONCE)
%     disp( 'local_findroot: *** search only within subset of corresponding sign')
%     keyboard
%     JUSTONCE = 0;
%   end
  
%   if isempty( max( abs( rt)) == max( ROOTS))
%     rt
%     keyboard
%     return
%   end
              
%  rtSRS = double( INVCARTAN) * double( rt);

%     HghtrtSRS = sum( rtSRS);
%     Sel = find( HghtrtSRS == HEIGHTOFROOTS);

%  global ROOTLEN

%   rtSRS = INVCARTAN * rt;
% 
%   Sel = find( sum( rtSRS) == HEIGHTOFROOTS);
%   if isempty( Sel)
%     rootIdx = 0;
%     return;
%   end

  
%   delta = sum( abs( rt(:,ones(1,length(Sel))) - ROOTS(:,Sel)), 1);
%   [MinVal, MinIdx] = min( delta);
%   if MinVal == 0,
%     rootIdx = Sel(MinIdx);
%   else
%     rootIdx = 0;
%   end

  NofRoots = size( ROOTS, 2);
  delta = sum( abs( rt(:,ones(1,NofRoots)) - ROOTS), 1);
  [MinVal, MinIdx] = min( delta);
  if MinVal == 0,
    rootIdx = MinIdx;
  else
    rootIdx = 0;
  end

  return

%   if 0
%     
%     numrts = size( ROOTS, 2);
% 
%     delta = sum( abs( rt(:,ones(1,numrts)) - ROOTS), 1);
% 
%     [MinVal, MinIdx] = min( delta);
%     if MinVal == 0,
%       rootIdx = MinIdx;
%   %    priv_assert( length( find( delta == 0)) == 1)
%     end
%     
%   end
  
end

% % -------------------------------------------------------------------------
% %
% %  find index into list of roots ROOTS corresponding to root rt
% %
% function rootIdx = local_findroot2( rt)
% 
% 
%   rootIdx = 0;
% 
%   global ROOTS
%   
%   numrts = size( ROOTS, 2);
%   
%   for i=1:numrts
%     if all( rt == ROOTS(:,i))
%       rootIdx = i;
%       return
%     end
%   end
%   
% %   return
% %     
% %   delta = sum( abs( rt(:,ones(1,numrts)) - ROOTS), 1);
% % 
% %   [MinVal, MinIdx] = min( delta);
% %   if MinVal == 0,
% %     rootIdx = MinIdx;
% % %    priv_assert( length( find( delta == 0)) == 1)
% %   end
%        
% end

% -------------------------------------------------------------------------
%
% calculate (alpha^i,alpha_i) = alpha_j g^ji alpha_i 
%
%
function local_initROOTLEN( metric)

  global ROOTS
  global ROOTLEN

  nofRoots = size( ROOTS, 2);

%  ROOTLEN = zeros( 1, nofRoots, 'int32');
  ROOTLEN = zeros( 1, nofRoots);

  for i = 1:nofRoots,
    ROOTLEN(i) = ROOTS(:,i)' * metric * ROOTS(:,i);
%    ROOTLEN(i) = double( ROOTS(:,i)') * double( metric) * double( ROOTS(:,i));
  end

end


% % -------------------------------------------------------------------------
% %
% %  Calculate structure constants N_r,s recursively.
% %
% %  Source:
% %
% %  R. J. Riebeek; Computations in Associations Schemes
% 
% function ret = local_structconst_sav( r, s)
% 
%   global SUMOFROOTS
%   global SIGNOFROOTS
%   global ISEXTRASPECIALPAIR
%   global ISSPECIALPAIR
%   global P
%   global ROOTFLIPSIGN
%   global ROOTLEN
%   global EXTRASPECIALPAIR
%   global ROOTSORDER
%   
% %   global NOFFUNCTIONCALL
% % 
% %   NOFFUNCTIONCALL = NOFFUNCTIONCALL+1;
%   
%   ret = 0;
% 
%   if r == 0,
% %    disp( [upper( mfilename) ': root r is zero root!'])
%     return;
%   end
%   
%   if s == 0,
% %    disp( [upper( mfilename) ': root s is zero root!'])
%     return;
%   end
%   
%   if SUMOFROOTS(r,s) == 0
%     return;
%   elseif SIGNOFROOTS(r) > 0 && ...
%          SIGNOFROOTS(s) > 0 && ...
%          ISEXTRASPECIALPAIR(r,s) > 0
% 
%     ret = P(r,s) + 1;
%     return
% 
%   elseif SIGNOFROOTS(r) < 0 && ...
%          SIGNOFROOTS(s) < 0 
% 
%     ret = -1 * local_structconstRiebeek( ROOTFLIPSIGN(r), ROOTFLIPSIGN(s)); 
% 
%   elseif ( SIGNOFROOTS(r) < 0 && SIGNOFROOTS(s) > 0 ) || ...
%          ( SIGNOFROOTS(s) < 0 && SIGNOFROOTS(r) > 0 )
%        
%     k = SUMOFROOTS(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s));
% 
% %    Sgn = SIGNOFROOTS(SUMOFROOTS(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s)));
%     Sgn = SIGNOFROOTS(k);
%     
%     if SIGNOFROOTS(r) == Sgn,
%       ret = local_structconstRiebeek( k, r) * ROOTLEN(k) / ROOTLEN(s);
%       return;
%     else
%       ret = local_structconstRiebeek( s, k) * ROOTLEN(k) / ROOTLEN(r);
%       return;      
%     end
%     
%   elseif ROOTSORDER(r) > ROOTSORDER(s) 
%     
%     ret = -1 * local_structconstRiebeek( s, r);
%     return;      
% 
%   else
%   
%     r1 = r;
%     r2 = s;
% 
% %    r
% %    s
% %    keyboard
%     
%     priv_assert( ISSPECIALPAIR(r1,r2))
%     
%     r1Pr2 = SUMOFROOTS(r1,r2);               % r1+r2
% 
%     Tmp = EXTRASPECIALPAIR(r1Pr2,:);
%     
%     r3 = Tmp(1);
%     r4 = Tmp(2);
%     r3Mr1 = SUMOFROOTS(r3,ROOTFLIPSIGN(r1)); % r3-r1 
%     r4Mr2 = SUMOFROOTS(r4,ROOTFLIPSIGN(r2)); % r4-r2 
%     r2Mr3 = SUMOFROOTS(r2,ROOTFLIPSIGN(r3)); % r2-r3 
%     r4Mr1 = SUMOFROOTS(r4,ROOTFLIPSIGN(r1)); % r4-r1 
%     
%     if r1Pr2,
%       r5Len = ROOTLEN(r1Pr2);
%     else
%       r5Len = 0;
%     end
%     
%     if r4Mr2,
%       r7Len = ROOTLEN(r4Mr2);
%     else
%       r7Len = 0;
%     end
%     
%     if r4Mr1,
%       r9Len = ROOTLEN(r4Mr1);
%     else
%       r9Len = 0;
%     end
%     
%     ret = ( r5Len * ...
%       ( local_structconstRiebeek(r3Mr1,r3) * local_structconstRiebeek(r4Mr2,r2) * ...
%         r7Len * ROOTLEN(r2) - ...
%         local_structconstRiebeek(r3,r2Mr3) * local_structconstRiebeek(r4Mr1,r1) * ...
%         r9Len * ROOTLEN(r1) ) ) / ...
%       ( local_structconstRiebeek(r3,r4) * ...
%         ROOTLEN(r1) * ROOTLEN(r2) * ROOTLEN(r4) );
% 
%     return;    
%     
%   end
%   
% end


% % -------------------------------------------------------------------------
% %
% %  initialize global variable HEIGHTOFROOTS
% %
% %  height of root : sum of root coordinates if expressed is linear
% %  combinarion of fundamental (simple) roots
% %  Thus, the height of fundamental (simple) root is unity.
% %
% function [heightOfRts, rootsSRS] = local_initHEIGHTOFROOTS( invcartan, cartan)
% 
%   global ROOTS
%   global ROOTLEN
% 
%   smplRootsIdx = local_findsimpleroots( ROOTS, cartan);
%   
% %   inv( roots(smplrootsidx,:)') * roots(1,:)'
% %   roots(smplrootsidx,:)' \ roots(1,:)'  
% %   inv( roots(smplrootsidx,:)')
%     
%   [Tmp, SortIdx] = sort( ROOTLEN(smplRootsIdx));
%   
% %   SortIdx
% %   keyboard
%   
% %  Tmp = invcartan * roots;
%   Tmp = invcartan * ROOTS;
%   rootsSRS = Tmp(SortIdx,:);
% 
%   heightOfRts = sum( rootsSRS, 1);
%   priv_assert( size( ROOTS, 2) == length( heightOfRts))
% 
%   persistent JUSTONCE
%   
%   if isempty( JUSTONCE)
%     disp( 'local_initHEIGHTOFROOTS: heightOfRts is ordered like rootsSRS, not ROOTS!!!')   
%     JUSTONCE = 1;
%     keyboard
%   end
%   
% %  heightOfRts
% %  rootsSRS
% %  keyboard
% 
% end

% -------------------------------------------------------------------------
%
%  Calculate structure constants N_r,s recursively.
%
%  Source:
%
%    R. J. Riebeek; Computations in Association Schemes
%
function ret = local_structconstRiebeek( r, s)

  global SUMOFROOTS
  global SIGNOFROOTS
  global ISEXTRASPECIALPAIR
  global PCOEFF
  global ROOTFLIPSIGN
  global ROOTLEN
  global EXTRASPECIALPAIR
%  global ROOTSORDER

%   if r == 1 && s == 9
%     keyboard
%   end
  
%  global ISSPECIALPAIR

%   global NOFFUNCTIONCALL
% 
%   NOFFUNCTIONCALL = NOFFUNCTIONCALL+1;
  

%   if r == 0,
% %    disp( [upper( mfilename) ': root r is zero root!'])
%     return;
%   end
%   
%   if s == 0,
% %    disp( [upper( mfilename) ': root s is zero root!'])
%     return;
%   end
  
  if SUMOFROOTS(r,s) == 0
    
    ret = 0;
    return
    
  elseif SIGNOFROOTS(r) > 0 && ...
         SIGNOFROOTS(s) > 0 && ...
         ISEXTRASPECIALPAIR(r,s) > 0

    ret = PCOEFF(r,s) + 1;

  elseif SIGNOFROOTS(r) < 0 && ...
         SIGNOFROOTS(s) < 0 

%
%  since N(r,s) = -N(-r,-s)
%
    ret = -1 * local_structconstRiebeek( ROOTFLIPSIGN(r), ROOTFLIPSIGN(s)); 
    
  elseif ( SIGNOFROOTS(r) < 0 && SIGNOFROOTS(s) > 0 ) || ...
         ( SIGNOFROOTS(s) < 0 && SIGNOFROOTS(r) > 0 )

% root k = -r - s       

    k = SUMOFROOTS(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s));

%    Sgn = SIGNOFROOTS(SUMOFROOTS(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s)));
    Sgn = SIGNOFROOTS(k);
    
    if SIGNOFROOTS(r) == Sgn,

      ret = local_structconstRiebeek( k, r) * ROOTLEN(k) / ROOTLEN(s);
      
    else
      
      ret = local_structconstRiebeek( s, k) * ROOTLEN(k) / ROOTLEN(r);

    end

     
%  elseif ROOTSORDER(r) > ROOTSORDER(s) 
  elseif r > s 
    
    ret = -1 * local_structconstRiebeek( s, r);

  else
  
    r1 = r;
    r2 = s;
%     r2 = r;
%     r1 = s;

%    priv_assert( ISSPECIALPAIR(r1,r2))
    
    r1Pr2 = SUMOFROOTS(r1,r2);               % r1+r2

%     Tmp = EXTRASPECIALPAIR(r1Pr2,:);
%     r3 = Tmp(1);
%     r4 = Tmp(2);
    r3 = EXTRASPECIALPAIR(r1Pr2,1);
    r4 = EXTRASPECIALPAIR(r1Pr2,2);
%     r4 = EXTRASPECIALPAIR(r1Pr2,1);
%     r3 = EXTRASPECIALPAIR(r1Pr2,2);
    
    r3Mr1 = SUMOFROOTS(r3,ROOTFLIPSIGN(r1)); % r3-r1 
    r4Mr2 = SUMOFROOTS(r4,ROOTFLIPSIGN(r2)); % r4-r2 
    r2Mr3 = SUMOFROOTS(r2,ROOTFLIPSIGN(r3)); % r2-r3 
    r4Mr1 = SUMOFROOTS(r4,ROOTFLIPSIGN(r1)); % r4-r1 

%     r1Mr3 = SUMOFROOTS(r3,ROOTFLIPSIGN(r1)); % r1-r3 
%     r2Mr4 = SUMOFROOTS(r4,ROOTFLIPSIGN(r2)); % r2-r4 
%     r3Mr2 = SUMOFROOTS(r2,ROOTFLIPSIGN(r3)); % r3-r2 
%     r1Mr4 = SUMOFROOTS(r4,ROOTFLIPSIGN(r1)); % r1-r4     
%     keyboard
    
    if r1Pr2 == 0,
      ret = 0;
      return
    end
    
    if r3Mr1 && r4Mr2,
      Tmp1 = local_structconstRiebeek(r3Mr1,r3) * ...
             local_structconstRiebeek(r4Mr2,r2) * ...
             ROOTLEN(r4Mr2) * ROOTLEN(r2);
%       Tmp1 = local_structconstRiebeek(r3Mr1,r3) * ...
%              local_structconstRiebeek(r4Mr2,r2) * ...
%              ROOTLEN(r4Mr2)^2 * ROOTLEN(r2)^2;
    else
      Tmp1 = 0;
    end
    
    if r2Mr3 && r4Mr1,
      Tmp2 = local_structconstRiebeek(r3,r2Mr3) * ...
             local_structconstRiebeek(r4Mr1,r1) * ...
             ROOTLEN(r4Mr1) * ROOTLEN(r1);
%       Tmp2 = local_structconstRiebeek(r3,r2Mr3) * ...
%              local_structconstRiebeek(r4Mr1,r1) * ...
%              ROOTLEN(r4Mr1)^2 * ROOTLEN(r1)^2;
    else
      Tmp2 = 0;
    end

    if Tmp1 && Tmp2,
      ret = ( ROOTLEN(r1Pr2) * ( Tmp1 - Tmp2 ) ) / ...
          ( local_structconstRiebeek(r3,r4) * ...
            ROOTLEN(r1) * ROOTLEN(r2) * ROOTLEN(r4) );
    else
      ret = 0;
    end
%     ret = ( ROOTLEN(r1Pr2)^2 * ( Tmp1 - Tmp2 ) ) / ...
%           ( local_structconstRiebeek(r3,r4) * ...
%             ROOTLEN(r1)^2 * ROOTLEN(r2)^2 * ROOTLEN(r4)^2 );
          
%     ret = ( r5Len * ...
%       ( local_structconstRiebeek(r3Mr1,r3) * local_structconstRiebeek(r4Mr2,r2) * ...
%         r7Len * ROOTLEN(r2) - ...
%         local_structconstRiebeek(r3,r2Mr3) * local_structconstRiebeek(r4Mr1,r1) * ...
%         r9Len * ROOTLEN(r1) ) ) / ...
%       ( local_structconstRiebeek(r3,r4) * ...
%         ROOTLEN(r1) * ROOTLEN(r2) * ROOTLEN(r4) );
    
  end

%   if ret == 0,
%     keyboard
%   end
  
end


% % -------------------------------------------------------------------------
% %
% %  Calculate structure constants N_r,s recursively.
% %
% %  Source:
% %
% %    Howlett; Matrix generators for exceptional groups of Lie type, 1999
% %
% function ret = local_structconstHowlettOld( a, b)
% 
%   global SUMOFROOTS
%   global SIGNOFROOTS
% %  global ISEXTRASPECIALPAIR
%   global PCOEFF
%   global ROOTFLIPSIGN
%   global ROOTLEN
%   global EXTRASPECIALPAIR
% %  global ROOTSORDER
%     
%   global HEIGHTOFROOTS
% 
% %   if a == 1 && b == 9
% %     keyboard
% %   end
%   
%   if SUMOFROOTS(a,b) == 0
%     
%     ret = 0;
%     return;
%       
%   elseif HEIGHTOFROOTS(a) > HEIGHTOFROOTS(b)
% 
%     ret = -1 * local_structconstHowlettOld( b, a);
%     return;
%     
%   elseif SIGNOFROOTS(a) < 0 
% 
%     if SIGNOFROOTS(b) < 0 
% 
%       ret = local_structconstHowlettOld( ROOTFLIPSIGN(b), ROOTFLIPSIGN(a)); 
%       return;
% 
%     else
%       
%       % b > 0
%       xi = SUMOFROOTS(a,b); 
%       if SIGNOFROOTS(xi) < 0
% 
%         ret = local_structconstHowlettOld( b, ROOTFLIPSIGN(xi)) * ...
%           ROOTLEN(xi) / ROOTLEN(a); 
%         return;
%         
%       elseif SIGNOFROOTS(xi) > 0
%         
%         % a+b > 0
%         ret = local_structconstHowlettOld( ROOTFLIPSIGN(a), xi) * ...
%           ROOTLEN(xi) / ROOTLEN(b); 
%         return;
%         
%       end
%       
%     end
%     
%   else
%     
% % a > 0    
%     aPb = SUMOFROOTS(a,b); 
%     Tmp = EXTRASPECIALPAIR(aPb,:);
%     
%     epsi = Tmp(1);
%     eta  = Tmp(2);
%     
%     if epsi == a
%       
%       ret = PCOEFF(a,b)+1;
%       return
%       
%     elseif epsi == b
%       
%       ret = -1;
%       return
%       
%     else
%       
%       bMepsi = SUMOFROOTS(b,ROOTFLIPSIGN(epsi));
%       if bMepsi == 0,
%         Tmp1 = 0;
% %        ret  = 0;
% %        return
%       else
%         Tmp1 = local_structconstHowlettOld( ROOTFLIPSIGN(epsi), b) * ...
%                local_structconstHowlettOld( ROOTFLIPSIGN(eta), a) / ...
%                ROOTLEN( bMepsi);
%       end
%       
%       aMepsi = SUMOFROOTS(a,ROOTFLIPSIGN(epsi));
%       if aMepsi == 0,
%         Tmp2 = 0;
% %         ret  = 0;
% %         return
%       else
%         Tmp2 = local_structconstHowlettOld( ROOTFLIPSIGN(epsi), a) * ...
%                local_structconstHowlettOld( ROOTFLIPSIGN(eta), b) / ...
%                ROOTLEN( aMepsi); 
%       end
%       
% %      ret = ROOTLEN(aPb) / (PCOEFF(a,b)+1) * (Tmp1 - Tmp2); 
%       ret = ROOTLEN(aPb) / (PCOEFF(epsi,eta)+1) * (Tmp1 - Tmp2); 
%       return
%       
%     end
%     
%   end
%   
% end

% % -------------------------------------------------------------------------
% %
% %  Calculate structure constants N_r,s recursively.
% %
% %  Source:
% %
% %    Howlett; Matrix generators for exceptional groups of Lie type, 1999
% %
% function ret = local_structconstHowlett( a, b, SUMOFROOTS, SIGNOFROOTS, ...
%   PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, HEIGHTOFROOTS)
% 
%   global NCOEFFHOWLETT_OLD
% %  global NCOEFFHOWLETTCALLS
%   
%   if ~isnan( NCOEFFHOWLETT_OLD(a,b))
% % no need to repeat calculation if we've already been here    
%     ret = NCOEFFHOWLETT_OLD(a,b);
%     return
%   end
%     
%   if SUMOFROOTS(a,b) == 0
%     
%     ret = 0;
%       
%   elseif HEIGHTOFROOTS(a) > HEIGHTOFROOTS(b)
% 
%     ret = -1 * local_structconstHowlett( b, a, SUMOFROOTS, SIGNOFROOTS, ...
%       PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, HEIGHTOFROOTS);
%     return;
%     
%   elseif SIGNOFROOTS(a) < 0 
% 
%     if SIGNOFROOTS(b) < 0 
% 
%       ret = local_structconstHowlett( ROOTFLIPSIGN(b), ROOTFLIPSIGN(a), ...
%         SUMOFROOTS, SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, ...
%         EXTRASPECIALPAIR, HEIGHTOFROOTS); 
% 
%     else
%       
%       % b > 0
%       xi = SUMOFROOTS(a,b); 
%       if SIGNOFROOTS(xi) < 0
% 
%         ret = local_structconstHowlett( b, ROOTFLIPSIGN(xi), SUMOFROOTS, ...
%           SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, ...
%           HEIGHTOFROOTS) * ROOTLEN(xi) / ROOTLEN(a); 
%         
%       elseif SIGNOFROOTS(xi) > 0
%         
%         % a+b > 0
%         ret = local_structconstHowlett( ROOTFLIPSIGN(a), xi, SUMOFROOTS, ...
%           SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, ...
%           HEIGHTOFROOTS) * ROOTLEN(xi) / ROOTLEN(b); 
%         
%       end
%       
%     end
%     
%   else
%     
% % a > 0    
%     aPb = SUMOFROOTS(a,b); 
% %     Tmp = EXTRASPECIALPAIR(aPb,:);
% %     epsi = Tmp(1);
% %     eta  = Tmp(2);
%     epsi = EXTRASPECIALPAIR(aPb,1);
%     eta  = EXTRASPECIALPAIR(aPb,2);
%     
%     if epsi == a
%       
%       ret = PCOEFF(a,b)+1;
%       
%     elseif epsi == b
%       
%       ret = -1;
%       
%     else
%       
%       bMepsi = SUMOFROOTS(b,ROOTFLIPSIGN(epsi));
%       
%       if bMepsi == 0,
%         Tmp1 = 0;
%       else
%         Tmp1 = local_structconstHowlett( ROOTFLIPSIGN(epsi), b, ...
%           SUMOFROOTS, SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, ...
%           EXTRASPECIALPAIR, HEIGHTOFROOTS) * ...
%                local_structconstHowlett( ROOTFLIPSIGN(eta), a, ...
%                SUMOFROOTS, SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, ...
%                EXTRASPECIALPAIR, HEIGHTOFROOTS) / ROOTLEN( bMepsi);
%       end
%       
%       aMepsi = SUMOFROOTS(a,ROOTFLIPSIGN(epsi));
%       
%       if aMepsi == 0,
%         Tmp2 = 0;
%       else
%         Tmp2 = local_structconstHowlett( ROOTFLIPSIGN(epsi), a, ...
%           SUMOFROOTS, SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, ...
%           EXTRASPECIALPAIR, HEIGHTOFROOTS) * ...
%                local_structconstHowlett( ROOTFLIPSIGN(eta), b, ...
%                SUMOFROOTS, SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, ...
%                EXTRASPECIALPAIR, HEIGHTOFROOTS) / ROOTLEN( aMepsi); 
%       end
%       
% %      ret = ROOTLEN(aPb) / (PCOEFF(a,b)+1) * (Tmp1 - Tmp2); 
%       ret = ROOTLEN(aPb) / (PCOEFF(epsi,eta)+1) * (Tmp1 - Tmp2); 
%       
%     end
%     
%   end
% 
% %
% %  store in global variable as well
% %
%   NCOEFFHOWLETT_OLD(a,b) = ret;
% 
% end

% -------------------------------------------------------------------------
%
%  plot root system
%
function local_plotrootsystem2d3d( invcartan, cartan)

  global ROOTS

  rank = size( ROOTS, 1);
 
  global SIGNOFROOTS

  rootsSRS = invcartan * ROOTS;
  
  posRootsIdx = find( SIGNOFROOTS > 0);
  negRootsIdx = find( SIGNOFROOTS < 0);
  
%  global CARTAN
  
  smplRootsIdx = local_findsimpleroots( ROOTS, cartan);

  if rank == 2,
    xMax = max( roots(1,:))+0.5;
    yMax = max( roots(2,:))+0.5;
    priv_clfig(1)
    plot(NaN)
    set( gca, 'dataaspect',[1 1 1])
    set( gca, 'xlim', [-xMax, xMax])
    set( gca, 'ylim', [-yMax, yMax])
    hold on
    local_arrowplot( roots(:,posRootsIdx), 'b');
    local_arrowplot( roots(:,smplRootsIdx), 'r');
    local_arrowplot( roots(:,negRootsIdx), 'g');
  end
  if rank == 3,
    nofRoots = size( roots, 2);
  %
  %  find positive roots
  %
    Tmp = zeros( 1, nofRoots);
    for i = 1:nofRoots,
      PosIdx = find( rootsSRS(:,i) > 0, 1);
      if isempty( PosIdx)
        continue
      end
      NegIdx = find( rootsSRS(:,i) < 0, 1);
      if isempty( NegIdx) || ~isempty( NegIdx) && PosIdx < NegIdx,
        Tmp(i) = 1;
      end
    end
    posRootsIdx = find( Tmp); 

  %
  %  find negative roots
  %
    Tmp = zeros( 1, nofRoots);
    for i = 1:nofRoots,
      NegIdx = find( rootsSRS(:,i) < 0, 1);
      if isempty( NegIdx)
        continue
      end
      PosIdx = find( rootsSRS(:,i) > 0, 1);
      if isempty( PosIdx) || ~isempty( PosIdx) && NegIdx < PosIdx,
        Tmp(i) = 1;
      end
    end
    negRootsIdx = find( Tmp);
    xMax = max( roots(1,:))+0.5;
    yMax = max( roots(2,:))+0.5;
    zMax = max( roots(3,:))+0.5;
    priv_clfig(1)
    plot3(NaN,NaN,NaN)
    hold on
%       local_arrowplot( roots(:,posRootsIdx), 'b');
%       local_arrowplot( roots(:,smplrootsidx), 'r');
%       local_arrowplot( roots(:,negRootsIdx), 'g');
    zer = zeros( size( posRootsIdx));
    line( [zer; roots(1,posRootsIdx)],  ...
          [zer; roots(2,posRootsIdx)],  ...
          [zer; roots(3,posRootsIdx)], 'color', 'b');
    line( [zer; roots(1,negRootsIdx)],  ...
          [zer; roots(2,negRootsIdx)],  ...
          [zer; roots(3,negRootsIdx)], 'color', 'g');
    zer = zeros( size( smplRootsIdx));
    line( [zer; roots(1,smplRootsIdx)],  ...
          [zer; roots(2,smplRootsIdx)],  ...
          [zer; roots(3,smplRootsIdx)], 'color', 'r');
    set( gca, 'xlim', [-xMax, xMax])
    set( gca, 'ylim', [-yMax, yMax])
    set( gca, 'zlim', [-zMax, zMax])
    set( gca, 'dataaspect',[1 1 1])
  end
  keyboard

end

% -------------------------------------------------------------------------
%
%  calculate N_{r,s}
%
function   NCoeff = local_assembleNCoeffRiebeek()

  global ROOTS

  nofRoots = size( ROOTS, 2);

  NCoeff = zeros( nofRoots, nofRoots);

  if 1,
    
    tic
    for i = 1:nofRoots,
      for j = i+1:nofRoots,
        NCoeff(i,j) = local_structconstRiebeek( i, j);
      end
    end
    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing local_structconstRiebeek() >' num2str( ElapsedTime) ' sec<'])

    for i = 1:nofRoots,
      for j = 1:i-1,
  %      TmpNCoeff(i,j) = -TmpNCoeff(j,i);
        NCoeff(i,j) = -NCoeff(j,i);
      end
    end
    
  else
    tic
    for i = 1:nofRoots,
      for j = 1:nofRoots,
        NCoeff(i,j) = local_structconstRiebeek( i, j);
      end
    end
    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing local_structconstRiebeek() >' num2str( ElapsedTime) ' sec<'])
  end

  NCUniq = sort( unique( NCoeff(:)))';
  
  NCUniq
  
  [NofNCOccurrence,Tmp] = hist( NCoeff(:), NCUniq);
  
  if ~isempty( NofNCOccurrence)
    NofNCOccurrence
  end
  
end

% % -------------------------------------------------------------------------
% %
% %
% %
% %
% function NCoeffHowlett = local_assembleNCoeffHowlettOld()
% 
%   global ROOTS
%   
%   nofRoots = size( ROOTS, 2);
% 
%   NCoeffHowlett = zeros(nofRoots,nofRoots);
% 
%   tic
%   for i = 1:nofRoots,
%     for j = i+1:nofRoots,
%       NCoeffHowlett(i,j) = local_structconstHowlettOld( i, j);
% %       if rem( NCoeffHowlett(i,j), 1)
% %         NCoeffHowlett(i,j)
% %         [i,j]
% %         keyboard
% %         local_structconstHowlettOld( i, j)
% %       end
%     end
%   end
%   ElapsedTime = toc;
%   disp( [upper( mfilename) ': timing local_structconstHowlettOld() >' num2str( ElapsedTime) ' sec<'])
% 
% %   global SUMOFROOTS
% %   global SIGNOFROOTS
% %   global PCOEFF
% %   global ROOTFLIPSIGN
% %   global ROOTLEN
% %   global EXTRASPECIALPAIR
% %   global HEIGHTOFROOTS
% % 
% %   NCoeffHowlett2 = zeros(nofRoots,nofRoots);
% % 
% %   tic
% %   for i = 1:nofRoots,
% %     for j = i+1:nofRoots,
% %       NCoeffHowlett2(i,j) = local_structconstHowlett( i, j, SUMOFROOTS, ...
% %         SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, HEIGHTOFROOTS);
% %       if rem( NCoeffHowlett2(i,j), 1)
% %         NCoeffHowlett(i,j)
% %         [i,j]
% %         keyboard
% % %        local_structconstHowlettOld( i, j)
% %       end
% %     end
% %   end
% %   toc
% %   priv_assert( all( NCoeffHowlett2 == NCoeffHowlett))
% %   keyboard
%   
%   for i = 1:nofRoots,
%     for j = 1:i-1,
%       NCoeffHowlett(i,j) = -NCoeffHowlett(j,i);
%     end
%   end
%   
% %   NCoeffHowlettSrt = NCoeffHowlett(ROOTSORDER,ROOTSORDER);
% %   priv_assert( NCoeffRiebeekSrt == NCoeffHowlettSrt)
% 
% end

% -------------------------------------------------------------------------
%
%
%
%
function NCoeffHowlett = local_assembleNCoeffHowlett()

  global ROOTS
  global SUMOFROOTS
  global SIGNOFROOTS
  global PCOEFF
  global ROOTFLIPSIGN
  global ROOTLEN
  global EXTRASPECIALPAIR
  global HEIGHTOFROOTS

  nofRoots = size( ROOTS, 2);

%  NCoeffHowlett = zeros(nofRoots,nofRoots);

%   global NCOEFFHOWLETT_OLD
%  
%   NCOEFFHOWLETT_OLD = zeros(nofRoots,nofRoots) * NaN;
% 
% %
% %  N(r,s) = -N(s,r) thus diagonal is zero
% %
%   for i = 1:nofRoots,
%     NCOEFFHOWLETT_OLD(i,i) = 0;
%   end
%   
% %  global NCOEFFHOWLETTCALLS
% %  NCOEFFHOWLETTCALLS = 0;


%   tic
%   for i = 1:nofRoots,
%     for j = i+1:nofRoots,
% 
%       NCOEFFHOWLETT_OLD(i,j) = local_structconstHowlett( i, j, SUMOFROOTS, ...
%         SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, HEIGHTOFROOTS);
% 
% %       Tmp = local_structconstRiebeek( i, j);
% %       if Tmp ~= NCOEFFHOWLETT_OLD(i,j)
% %         Tmp
% %         NCOEFFHOWLETT_OLD(i,j)
% %         keyboard
% %         Tmp = local_structconstRiebeek( i, j);
% %         Tmp = local_structconstHowlett( i, j, SUMOFROOTS, ...
% %         SIGNOFROOTS, PCOEFF, ROOTFLIPSIGN, ROOTLEN, EXTRASPECIALPAIR, HEIGHTOFROOTS);
% %       end
%       
%     end
%   end
%   ElapsedTime = toc;
%   disp( [upper( mfilename) ': timing local_structconstHowlett() >' num2str( ElapsedTime) ' sec<'])

  
  NCoeffHowlett = zeros(nofRoots,nofRoots) * NaN;
%   NCoeffInit = 3*nofRoots;
%   NCoeffHowlett = ones( nofRoots, nofRoots, 'int16') * NCoeffInit;

%
%  N(r,s) = -N(s,r) thus diagonal is zero
%
  for i = 1:nofRoots,
    NCoeffHowlett(i,i) = 0;
  end

  
  %
  %  Calculate structure constants N_r,s recursively.
  %
  %  Source:
  %
  %    Howlett; Matrix generators for exceptional groups of Lie type, 1999
  %
  function ret = nested_structconstHowlett( a, b)

    if ~isnan( NCoeffHowlett(a,b))
%    if NCoeffHowlett(a,b) ~= NCoeffInit
  % no need to repeat calculation if we've already been here    
      ret = NCoeffHowlett(a,b);
      return
    end

    if SUMOFROOTS(a,b) == 0
      ret = 0;
    elseif HEIGHTOFROOTS(a) > HEIGHTOFROOTS(b)
      ret = -1 * nested_structconstHowlett( b, a);
      return;
    elseif SIGNOFROOTS(a) < 0 

      if SIGNOFROOTS(b) < 0 
        ret = nested_structconstHowlett( ROOTFLIPSIGN(b), ROOTFLIPSIGN(a)); 
      else

        % b > 0
        xi = SUMOFROOTS(a,b); 
        if SIGNOFROOTS(xi) < 0

          ret = ( nested_structconstHowlett( b, ROOTFLIPSIGN(xi)) * ...
                  ROOTLEN(xi)) / ROOTLEN(a); 

        elseif SIGNOFROOTS(xi) > 0

          % a+b > 0
          ret = ( nested_structconstHowlett( ROOTFLIPSIGN(a), xi) * ...
                  ROOTLEN(xi)) / ROOTLEN(b); 

        end
      end
    else

  % a > 0    
      aPb = SUMOFROOTS(a,b); 
      epsi = EXTRASPECIALPAIR(aPb,1);
      eta  = EXTRASPECIALPAIR(aPb,2);

      if epsi == a
        ret = PCOEFF(a,b)+1;
      elseif epsi == b
        ret = -1;
      else

        bMepsi = SUMOFROOTS(b,ROOTFLIPSIGN(epsi));

        if bMepsi == 0,
          Tmp1 = 0;
        else
          Tmp1 = ( nested_structconstHowlett( ROOTFLIPSIGN(epsi), b) * ...
                   nested_structconstHowlett( ROOTFLIPSIGN(eta), a)) / ...
                 ROOTLEN( bMepsi);
        end

        aMepsi = SUMOFROOTS(a,ROOTFLIPSIGN(epsi));

        if aMepsi == 0,
          Tmp2 = 0;
        else
          Tmp2 = ( nested_structconstHowlett( ROOTFLIPSIGN(epsi), a) * ...
                   nested_structconstHowlett( ROOTFLIPSIGN(eta), b)) / ...
                 ROOTLEN( aMepsi); 
        end

  %      ret = ROOTLEN(aPb) / (PCOEFF(a,b)+1) * (Tmp1 - Tmp2); 
        ret = (ROOTLEN(aPb) * (Tmp1 - Tmp2)) / (PCOEFF(epsi,eta)+1); 

      end
    end

%
%  if in recurive call, store this result anyway. we may need it again...
%
    NCoeffHowlett(a,b) = ret;

  end
  % --- end nested function 
    
% structure constants are antisymmetric. We loop over one triangle only.  
  tic
  for i = 1:nofRoots,
    for j = i+1:nofRoots,
      NCoeffHowlett(i,j) = nested_structconstHowlett( i, j);      
    end
  end
  ElapsedTime = toc;
  disp( [upper( mfilename) ': timing nested_structconstHowlett() >' ...
    num2str( ElapsedTime) ' sec<'])
  
%  NCOEFFHOWLETTCALLS
%  keyboard
%  priv_assert( all( NCoeffHowlett2 == NCoeffHowlett))  
%  keyboard

%
%  structure constants are antisymmetric. so we just copy from other
%  triangle
%
  for i = 1:nofRoots,
    for j = 1:i-1,
%      NCOEFFHOWLETT_OLD(i,j) = -NCOEFFHOWLETT_OLD(j,i);
      NCoeffHowlett(i,j) = -NCoeffHowlett(j,i);
    end
  end
  
%  priv_assert( all( NCOEFFHOWLETT_OLD == NCOEFFHOWLETT))
    
%   NCoeffHowlettSrt = NCoeffHowlett(ROOTSORDER,ROOTSORDER);
%   priv_assert( NCoeffRiebeekSrt == NCoeffHowlettSrt)

end

% -------------------------------------------------------------------------
%
%  validate roots
%
function local_checkroots( cartan, metric)

  global ROOTS
  global SIGNOFROOTS
  global SUMOFROOTS
  global ROOTFLIPSIGN
  global INVCARTAN
  global ROOTSRS

%   ROOTS
%   cartan
%   keyboard
  
  priv_assert( ~isempty( SIGNOFROOTS))
  
%
%  find simple roots
%

  numRoots = size( ROOTS, 2);
  rank     = size( ROOTS, 1);

%
%  in ROOTSRS simple roots should appear as (multiple of) unit 
%  (rank x rank) matrix (column-reversed order)
%
  Tmp = ROOTSRS(:,end/2+rank:-1:end/2+1);
  priv_assert( all( all( Tmp(1,1)*eye( rank) == Tmp)))  

  
  smplRootsIdx = local_findsimpleroots( ROOTS, cartan);
  
%   smplRootsIdx
%   keyboard

%  numSmplRoots = size( cartan, 2);

  rootsSRS = INVCARTAN * ROOTS;
%  rootsSRS = double( INVCARTAN) * double( ROOTS);

  Tmp = rootsSRS(:,end/2+rank:-1:end/2+1);

  priv_assert( all( all( Tmp(1,1)*eye( rank) == Tmp)))  
  
  if Tmp(1,1) ~= 1
%    disp( [upper( mfilename) ': normalize rootsSRS by ' num2str( Tmp(1,1))])
    disp( [upper( mfilename) ...
      ': normalization of rootSRS     = ' num2str( Tmp(1,1))])
  end
  
  Tmp = Tmp / Tmp(1,1);
  priv_assert( all( all( eye( rank) == Tmp)))  
  
  
  DiagSmplRoots = diag( rootsSRS(:,smplRootsIdx));
  priv_assert( diag( DiagSmplRoots) == rootsSRS(:,smplRootsIdx))
  
  rootsSRSNrm = rootsSRS ./ repmat( DiagSmplRoots, 1, numRoots);

  priv_assert( diag( DiagSmplRoots) == rootsSRS(:,smplRootsIdx))
  
  posRootIdx = ones( 1, numRoots/2) * NaN;
  k = 1;
  for i = 1:numRoots,
    if SIGNOFROOTS(i) < 0 
      continue
    end
    posRootIdx(k) = i;
    k = k+1;
  end
  
  priv_assert( all( rootsSRSNrm(:,posRootIdx) >= 0))

  posNonSimpleIdx = setdiff( posRootIdx, smplRootsIdx);
 
%
%  make sure that there is simple root such that 
%  non-simple root minus simple root is again a root
%
  for i = posNonSimpleIdx,
    IsRoot = 0;
    for j = smplRootsIdx,
      k = SUMOFROOTS( i, ROOTFLIPSIGN(j));
      if k,
        IsRoot = 1;
        break;
      end
    end
    priv_assert( IsRoot)
  end

  return
  
  
  
  for i = 1:numRoots
    RootLen(i) = ROOTS(:,i)' * metric * ROOTS(:,i);
  end
  
  RootLen
  keyboard

  RootMetricRoot   = zeros( numRoots, numRoots);
  RatioRootLengths = zeros( numRoots, numRoots) * NaN;
%  RootRoot       = zeros( numRoots, numRoots);

% FIXME: Cartan matrix is inverted column-wise ...
% ROOTS(:,SimpleRootsIdx) = cartan(:,end:-1:1)
%  Tmp = cartan;
  Tmp = ROOTS(:,smplRootsIdx);
  Tmp(Tmp==0) = NaN;
  CartanRatioFlipped = Tmp ./ Tmp';
  CartanRatio = CartanRatioFlipped(end:-1:1,end:-1:1);
  
%   [i,j] = find( RootMetricRootFlipped > 1);
%   priv_assert( length( i) <= 1)
%   
%   if length( i) == 1
%     if i > j
%       RootLengthChange = -1;
%       RootLengthRatio  = RootMetricRootFlipped(j,i);
%     else
%       RootLengthChange = 1;
%       RootLengthRatio  = RootMetricRootFlipped(i,j);
%     end
%     priv_assert( RootLengthRatio > 1)
%   else
%     RootLengthChange = 0;
%     RootLengthRatio  = 1;
%   end
  
%   RootLengthRatio
%   RootLengthChange
  
%  [i,j]
  
%   for i = 1:numRoots
%     for j = 1:numRoots
%       RootRoot(i,j) = ROOTS(:,i)' * ROOTS(:,j);
%     end
%   end

%  ROOTS
%  metric
%  cartan
  smplRootsIdx
  CartanRatio
  CartanRatioFlipped
%  ROOTS(:,smplRootsIdx)
  cartan
  
%  RootMetricRoot
%  RootRoot

  keyboard










  return
  
  
  
  rootsSRSNrm
  keyboard

  for i = smplRootsIdx,
    j = find( rootsSRS(:,i));
    priv_assert( length( j))
    
  end
  keyboard
  
  
  x = ones( numRoots/2, numRoots/2) * NaN;

  for i = 1:numRoots,
    if SIGNOFROOTS(i) < 0 
      continue
    end
    NonSmplRoots = invcartan * ROOTS  
  end
  keyboard

  for i = 1:numRoots,
    for j = 1:numRoots,
      if SIGNOFROOTS(i) < 0 || SIGNOFROOTS(j) < 0 
        continue
      end
      x(i,j) = 2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
                   (ROOTS(:,j)' * metric * ROOTS(:,j));  
    end
  end
  
  x
  keyboard
  
  
  
  
  
  
  
  
  
%  keyboard
  
  j = 1;

  for i = 1:numRoots,
    Idx = find( rootsSRS(:,i));
    if length( Idx) ~= 1,
      continue
    end
    if rootsSRS(Idx,i) < 0,
      continue
    end
    smplRootsIdx(j) = i;
    j = j+1;
  end
  
  rootsSRS
  smplRootsIdx
  
  rootsSRS / rootsSRS(:,smplRootsIdx(1))
  
  keyboard
  
end

% -------------------------------------------------------------------------
%
% check corerctness of special root pairs
%
function local_checkspecialpairs()

  global ROOTS
  global SUMOFROOTS
  global SIGNOFROOTS
  global ROOTFLIPSIGN
  global ISSPECIALPAIR
  global EXTRASPECIALPAIR
%  global ROOTSORDER
  
  nofRoots = size( ROOTS, 2);

  x = zeros(1,12);
  for r = 1:nofRoots,
    for s = 1:nofRoots,

      if SUMOFROOTS(r,s) == 0
        continue
      end
      
      if SIGNOFROOTS(r) < 0 || SIGNOFROOTS(s) < 0
        continue
      end
      
      MrMs = SUMOFROOTS(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s));
      rPs  = SUMOFROOTS(r,s);

%
%  exactly one of those 12 combinations is a special pair
%  (Carter, p59)
%
      x(1)=ISSPECIALPAIR(r,s);
      x(2)=ISSPECIALPAIR(s,r);
      x(3)=ISSPECIALPAIR(s,MrMs);
      x(4)=ISSPECIALPAIR(MrMs,s);
      x(5)=ISSPECIALPAIR(r,MrMs);
      x(6)=ISSPECIALPAIR(MrMs,r);
      x(7)=ISSPECIALPAIR(ROOTFLIPSIGN(r),ROOTFLIPSIGN(s));
      x(8)=ISSPECIALPAIR(ROOTFLIPSIGN(s),ROOTFLIPSIGN(r));
      x(9)=ISSPECIALPAIR(ROOTFLIPSIGN(s),rPs);
      x(10)=ISSPECIALPAIR(rPs,ROOTFLIPSIGN(s));
      x(11)=ISSPECIALPAIR(ROOTFLIPSIGN(r),rPs);
      x(12)=ISSPECIALPAIR(rPs,ROOTFLIPSIGN(r));
      
%
%  r and s are special if r, s, r+s are positive roots and r < s
%
%  the special pair (r,s) is called extraspecial if for all other 
%  special pairs (r1,s1) with r1+s1=r+s we have r1 > r
%
      if length( find( x)) ~= 1
        disp( sprintf( 'r=%d  s=%d  -r=%d  -s=%d  -r-s=%d  r+s=%d', r, s, ...
          ROOTFLIPSIGN(r),ROOTFLIPSIGN(s), MrMs, rPs))
        x
        EXTRASPECIALPAIR'
%        ISSPECIALPAIR
        ROOTS
%        ROOTSORDER        
%        keyboard
      end
    end
  end
%  keyboard
end


% -------------------------------------------------------------------------
%
%  check correctness of structure constants
%
function local_checkNCoeff( NCoeff)

  global ROOTS
  global SUMOFROOTS

  global EXTRASPECIALPAIR
  
  nofRoots = size( ROOTS, 2);

  for r = 1:nofRoots,
    for s = 1:nofRoots,
      
      if SUMOFROOTS(r,s) <= 0
        continue
      end
      
      if abs( NCoeff(r,s)) < 1 || abs( NCoeff(r,s)) > 3
        SUMOFROOTS(r,s)
        [r,s]
        NCoeff
        keyboard
      end
      
    end
  end

%
%  we choose N(r,s) = +(p+1) for all extraspecial pairs
%
  for r = 1:nofRoots,
    if EXTRASPECIALPAIR(r,1) == 0 || ...
       EXTRASPECIALPAIR(r,2) == 0,
      continue;     
    end
    priv_assert( NCoeff(EXTRASPECIALPAIR(r,1),EXTRASPECIALPAIR(r,2)) > 0);
  end
  
end

% -------------------------------------------------------------------------
%
%  create color map for range of values
%
%    []                          A1
%    [-1, +1]                    A2,A3,... D4,D5,...
%    [-2, -1, +1, +2]            B2,B3,... C3,C4,...
%    [-3, -2, -1, +1, +2, +3]    F4
%
function cm = local_createcolormap( zVal)

%  x( isnan(x)) = [];
  
%
%  make sure 0 is colored white
%
  Idx = find( zVal ~= 0);
  cm = ones( length( zVal), 3);

  switch( length( Idx))
    case 1
      cm(Idx,:) = [0  0.7  0];

    case 2
      cm(Idx,:) = [1  0    0;
            0  0.7  0];

%     case 3
%       cm(Idx,:) = [1  0   0;
%             1  1   1;
%             0  0.7 0];

    case 4
      cm(Idx,:) = [0   0   1;
            1   0   0;
            0   0.7 0;
            0.6 0   0.6];

%     case 5
%       cm(Idx,:) = [0   0   1;
%             1   0   0;
%             1   1   1;
%             0   0.7 0;
%             0.6 0   0.6];

    case 6
      cm(Idx,:) = [0    0.5  0.5;
            0    0    1;
            1    0    0;
            0    0.7  0;
            0.6  0    0.6;
            0.7  0.7  0];

%     case 7
%       cm(Idx,:) = [0   0.5   0.5;
%             0    0    1;
%             1    0    0;
%             1    1    1;
%             0    0.7  0;
%             0.6  0    0.6;
%             0.7  0.7  0];
            
    otherwise
      cm = [];
      return
  end
  
end
