function priv_check4jacobiidentity( metric, NCoeff)
%PRIV_CHECK4JACOBIIDENTITY  check if Jacobi identity is fulfilled
%
%  priv_check4jacobiidentity( metric, NCoeff)
%

%
%  4-Jacobi identity :
%
%  [[[a,b],c],d] + [[[b,c],d],a] + [[[c,d],a],b] + [[[d,a],b],c] = ?
%

% (c) G. Beyerle 2012

  global ROOTS
  global SUMOFROOTSX
  global SUMOFROOTS

  global ROOTSORDER
  global PCOEFF
  global ROOTFLIPSIGN
  global SIGNOFROOTS
  
  priv_assert( ~isempty( SUMOFROOTSX))
  
  nofRoots = size( ROOTS, 2);

  CartanX = local_initCARTANX( metric);

  NR = nofRoots;

  NCoeffX = ones(2*NR+1,2*NR+1)*(-999);
  NCoeffX(1:NR,1:NR) = NCoeff;

  NCoeffX(NR+(1:NR),NR+(1:NR)) = 0;

  NCoeffX(NR+(1:NR),1:NR) = CartanX;
  NCoeffX(1:NR,NR+(1:NR)) = -CartanX';

%     NCoeffX
%     keyboard

  disp( [upper( mfilename) ': check Jacobi identity ...'])

% nested function  
  function jjStr = nested_tr(ii)
    if ii == 2*NR+1,
      jjStr = '0';
    elseif ii > NR && ii <= 2*NR,
      jjStr = ['z' num2str(ii)];
    else
      jjStr = num2str(ii);
    end
  end
  
  function Cof = nested_getcoeff2( i, j)
    if ( i == ROOTFLIPSIGN( j))
  % pos/neg root pair!    
      Cof  = 1 * SIGNOFROOTS(i);
  %    Cof  = 1;
    else
      Cof  = NCoeff(i,j);
    end
  end
  
  function Cof = nested_getcoeff2x( i, j)
    if ( i == ROOTFLIPSIGN( j))
  % pos/neg root pair!    
      Cof  = 1 * SIGNOFROOTS(i);
  %    Cof  = 1;
    else
      Cof  = NCoeffX(i,j);
    end
  end


  NofFailed = 0;

  for i = 1:nofRoots,
    for j = 1:nofRoots,
      for k = 1:nofRoots,
        for l = 1:nofRoots,

%         if i == 1 && j == 2 && k == 10,
%           jk = 13
%           local_getcoeff2( jk, i, NCoeffX)
%           jk = jk - NR;
%           2 * ROOTS(:,jk)' * metric * ROOTS(:,i) / ...
%               (ROOTS(:,i)' * metric * ROOTS(:,i))
%           2 * ROOTS(:,jk)' * metric * ROOTS(:,i) / ...
%              (ROOTS(:,jk)' * metric * ROOTS(:,jk))
%           keyboard
%         end

          ijCof = nested_getcoeff2( i, j);
          jkCof = nested_getcoeff2( j, k);
          klCof = nested_getcoeff2( k, l);
          liCof = nested_getcoeff2( l, i);

          ij     = SUMOFROOTSX(i,j);
          jk     = SUMOFROOTSX(j,k);
          kl     = SUMOFROOTSX(k,l);
          li     = SUMOFROOTSX(l,i);

  %          ijkCof = NCoeffX(ij,k);
  %          jkiCof = NCoeffX(jk,i);
  %          kijCof = NCoeffX(ki,j);
          ijkCof = nested_getcoeff2x( ij, k);
          jklCof = nested_getcoeff2x( jk, l);
          kliCof = nested_getcoeff2x( kl, i);
          lijCof = nested_getcoeff2x( li, j);

          ijk    = SUMOFROOTSX(ij,k);
          jkl    = SUMOFROOTSX(jk,l);
          kli    = SUMOFROOTSX(kl,i);
          lij    = SUMOFROOTSX(li,j);

          ijklCof = nested_getcoeff2x( ijk, l);
          jkliCof = nested_getcoeff2x( jkl, i);
          klijCof = nested_getcoeff2x( kli, j);
          lijkCof = nested_getcoeff2x( lij, k);

          ijkl    = SUMOFROOTSX(ijk,l);
          jkli    = SUMOFROOTSX(jkl,i);
          klij    = SUMOFROOTSX(kli,j);
          lijk    = SUMOFROOTSX(lij,k);

          Cof = [ijCof*ijkCof*ijklCof, jkCof*jklCof*jkliCof, ...
                 klCof*kliCof*klijCof, liCof*lijCof*lijkCof];
         
          Gen = [ijkl, jkli, klij, lijk];
          
          Sel = find( Cof);
          if isempty( Sel)
            continue
          end
          
          if all( Gen(Sel) == Gen(Sel(1))) && sum( Cof(Sel)) == 0
            continue
          end
            
          if ( kli == 2*nofRoots+1 && ...
               lij == 2*nofRoots+1 && ...
               ijk == 2*nofRoots+1 && ...
               lij == 2*nofRoots+1 )
  % all terms are zero             
            continue
          end

          if ( ~ijCof && ~jkCof && ~klCof && ~liCof )
  % all coefficients of 'inner' commutators terms are zero             
            continue
          end
          
          if ( ~ijkCof && ~jklCof && ~kliCof && ~lijCof )
  % all coefficients of 'inner' commutators terms are zero             
            continue
          end
          
          if ( ~ijklCof && ~jkliCof && ~klijCof && ~lijkCof )
  % all coefficients of 'inner' commutators terms are zero             
            continue
          end
          
          if ( ijCof * ijkCof * ijklCof == 0 && ...
               jkCof * jklCof * jkliCof == 0 && ...
               klCof * kliCof * klijCof == 0 && ...
               liCof * lijCof * lijkCof == 0 )
  % all coefficients of 'inner' commutators terms are zero             
            continue
          end

%           if ijCof*ijkCof == 0 && jkCof*jkiCof == -kiCof*kijCof && jki == kij
%             continue
%           end
%           if jkCof*jkiCof == 0 && kiCof*kijCof == -ijCof*ijkCof && kij == ijk
%             continue
%           end
%           if kiCof*kijCof == 0 && ijCof*ijkCof == -jkCof*jkiCof && ijk == jki
%             continue
%           end
%           
%           if (ijk <= nofRoots) && ijk == jki && jki == kij && ...
%               ijCof*ijkCof + jkCof*jkiCof + kiCof*kijCof == 0
%             continue
%           end
% 
%           if ( ijk >= nofRoots+1 && ijk <= 2*nofRoots && ...
%              jki >= nofRoots+1 && jki <= 2*nofRoots && ...
%              kij >= nofRoots+1 && kij <= 2*nofRoots )
%             continue
%           end

          disp( '-----------')
          disp( sprintf( 'i  = %2d, j  = %2d, k  = %2d, l  = %2d', i, j, k, l))
%           disp( sprintf( 'ij = %2d, jk = %2d, kl = %2d, li = %2d', ij, jk, kl, li))
%           disp( sprintf( 'ijk= %2d, jkl= %2d, kli= %2d, lij= %2d', ijk, jkl, kli, lij))
%          disp(' ')
%           disp( sprintf( '[[[%d,%d],%d],%d] + [[[%d,%d],%d],%d] + [[[%d,%d],%d],%d] + [[[%d,%d],%d],%d] = ', ...
%             i,j,k,l, j,k,l,i, k,l,i,j, l,i,j,k))
% 
%           disp( sprintf( '(%d)*[[%s,%d],%d] + (%d)*[[%s,%d],%d] + (%d)*[[%s,%d],%d] + (%d)*[[%s,%d],%d] = ', ...
%             ijCof,nested_tr(ij),k,l, jkCof,nested_tr(jk),l,i, ...
%             klCof,nested_tr(kl),i,j, liCof,nested_tr(li),j,k))
% 
%           disp( sprintf( '(%d)*(%d)*[%s,%d] + (%d)*(%d)*[%s,%d] + (%d)*(%d)*[%s,%d] + (%d)*(%d)*[%s,%d] = ', ...
%             ijCof,ijkCof,nested_tr(ijk),l, jkCof,jklCof,nested_tr(jkl),i, ...
%             klCof,kliCof,nested_tr(kli),j, liCof,lijCof,nested_tr(lij),k))
% 
%           disp( sprintf( '(%d)*(%d)*(%d)*[%s] + (%d)*(%d)*(%d)*[%s] + (%d)*(%d)*(%d)*[%s] + (%d)*(%d)*(%d)*[%s] = ', ...
%             ijCof,ijkCof,ijklCof,nested_tr(ijkl), jkCof,jklCof,jkliCof,nested_tr(jkli), ...
%             klCof,kliCof,klijCof,nested_tr(klij), liCof,lijCof,lijkCof,nested_tr(lijk)))

          disp( sprintf( '(%d)*[%s] + (%d)*[%s] + (%d)*[%s] + (%d)*[%s] ', ...
            ijCof*ijkCof*ijklCof,nested_tr(ijkl), jkCof*jklCof*jkliCof,nested_tr(jkli), ...
            klCof*kliCof*klijCof,nested_tr(klij), liCof*lijCof*lijkCof,nested_tr(lijk)))

%           disp( sprintf( '(%d)*(%d)*[%s] + (%d)*(%d)*[%s] + (%d)*(%d)*[%s] = ', ...
%             ijCof,ijkCof,nested_tr(ijk), jkCof,jkiCof,nested_tr(jki), kiCof,kijCof,nested_tr(kij)))
%           disp( sprintf( '(%d)*[%s] + (%d)*[%s] + (%d)*[%s]', ...
%             ijCof*ijkCof,nested_tr(ijk), jkCof*jkiCof,nested_tr(jki), kiCof*kijCof,nested_tr(kij)))
%           NCoeff
%           SUMOFROOTS
%   %        ROOTSORDER
%   %        SUMOFROOTSX
%   %        PCOEFF
%           ROOTSORDER
%           ROOTS

%          keyboard

          NofFailed = NofFailed+1; 
        end
      end
    end
  end
  
  disp( [upper( mfilename) ': ' num2str( NofFailed) ' failed cases.' ])
   
  if NofFailed
    disp( [upper( mfilename) ': ... 4-Jacobi identity invalid'])
  else
    disp( [upper( mfilename) ': ... 4-Jacobi identity valid'])
  end
  
  keyboard
  
end

% -------------------------------------------------------------------------
%
%
%
%
%
function CartanX = local_initCARTANX( metric)

  global ROOTS
              
  numrts = size( ROOTS, 2);

  CartanX = zeros( numrts, numrts);
  
  for i = 1:numrts,
    for j = 1:numrts,
      
      CartanX(i,j) =  2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
                          (ROOTS(:,i)' * metric * ROOTS(:,i));
%      CartanX(i,j) =  2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
%                          (ROOTS(:,j)' * metric * ROOTS(:,j));

%       CartanX(i,j) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
%                         (ROOTS(:,i)' * metric * ROOTS(:,i));
%       CartanX(i,j) =      (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
%                           (ROOTS(:,i)' * metric * ROOTS(:,i));
%       CartanX(j,i) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
%                         (ROOTS(:,i)' * metric * ROOTS(:,i));
    end
  end

end

% -------------------------------------------------------------------------
%
%
%
%
function Cof = local_getcoeff2( i, j, NCoeff)
        
  global ROOTFLIPSIGN
  global SIGNOFROOTS

  if ( i == ROOTFLIPSIGN( j))
% pos/neg root pair!    
    Cof  = 1 * SIGNOFROOTS(i);
%    Cof  = 1;
  else
    Cof  = NCoeff(i,j);
  end
  
end
  
