function priv_testadjointrepresentation( metric, NCoeff, cartan)
%PRIV_TESTADJOINTREPRESENTATION   check metric tensor
%
%  priv_validatemetrictensor( metric, NCoeff)
%

% (c) G. Beyerle 2012

  global ROOTS
  global ROOTSRS
  global SUMOFROOTS
   
  TestFailed = 0;

%
%  ordering of roots
%
%  i = 1,2,..,NofRoots,NofRoots+1,...,NofRoots+Rank
%
%  i = NofRoots/2+1,...,NofRoots/2+Rank : simple roots
%  i = NofRoots/2+1,...,NofRoots        : positive roots
%  i = 1,...,NofRoots/2                 : negative roots
%  i = 1 & i = NofRoots                 : pair of corresp. neg & pos roots
%  i = 2 & i = NofRoots-1               : pair of corresp. neg & pos roots
%  ...
%  i = NofRoots/2 & i = NofRoots/2+1    : pair of corresp. neg & pos roots
%

  %  metric
  
  disp( [upper( mfilename) ': checking adjoint representation ...'])

  NofRoots = size( ROOTS, 2);
  Rank     = size( ROOTS, 1); % dimension of Cartan subalgebra

%
% calculate fulls structure constants
%  
  N = NofRoots + Rank;

  StructConst = zeros( N, N, N);

  NofPosRoots          = NofRoots / 2;
  SimpleRootsIdx       = NofPosRoots + (1:Rank);
  NotSimplePosRootsIdx = SimpleRootsIdx(end)+(1:(NofPosRoots-Rank));
  ZeroRootsIdx         = NofRoots + (1:Rank);

%  
%   f^k_{i,j} = StructConst(i,j,k)
%   f^k_{i,j} = -f^k_{j,i}
%

%
%  consider
%
%    [E_{beta}, E_{gamma}]
%
%  a) beta+gamma is root
%
%    [E_{beta}, E_{gamma}] = N_{beta,gamma} * E_{beta+gamma} 
%
%  b) beta+gamma is not a root
%
%    [E_{beta}, E_{gamma}] = 0 
%
  for i = 1:NofRoots,
    for j = 1:NofRoots,
      k = SUMOFROOTS(i,j);
      if i + j == NofRoots+1,
% case [E_{beta}, E_{-beta}] is treated below        
        priv_assert( k == 0)
        continue
      end
      if k > 0,
        mlassert( StructConst(i,j,k) == 0) 
        StructConst(i,j,k) = NCoeff(i,j);
      end
    end
  end

  
%
%  consider
%
%    [E_{beta}, E_{-beta}] = H_{beta}
%
%  case a) : beta is a simple root
%
  for i = SimpleRootsIdx,
    k = i + NofRoots/2;
    mlassert( StructConst(i,NofRoots-i+1,k) == 0) 
    StructConst(i,NofRoots-i+1,k) = 1;
    mlassert( StructConst(NofRoots-i+1,i,k) == 0) 
    StructConst(NofRoots-i+1,i,k) = -1;
  end
  
%
%  consider
%
%    [E_{beta}, E_{-beta}] = H_{beta}
%
%  case b) : beta is _not_ a simple root
%

%
%  first find coefficients of linear expansion of non-simple roots
%  in terms of simple roots
%
     
  RootCoeff = ROOTSRS;
  for i = SimpleRootsIdx,
    Idx = find( RootCoeff(:,i));
    mlassert( length( Idx) == 1);
    mlassert( RootCoeff(Idx,i) > 0 );
    RootCoeff(Idx,:) = RootCoeff(Idx,:) / RootCoeff(Idx,i);
  end
  
%   NotSimplePosRootsIdx
%   keyboard 

  for i = NotSimplePosRootsIdx,
%    kFct = RootCoeff(end:-1:1,i); % reverse order in rank (first) coordinate!
    kFct = RootCoeff(:,i); 
    for l = 1:Rank,
      Tmp = (ROOTS(:,l)' * metric * ROOTS(:,l)) / ...
            (ROOTS(:,i)' * metric * ROOTS(:,i));
      k = NofRoots+l;
      mlassert( StructConst(i,NofRoots-i+1,k) == 0) 
      StructConst(i,NofRoots-i+1,k) = kFct(l) * Tmp;
% ... the corresponding (negative) value at the mirror location
      mlassert( StructConst(NofRoots-i+1,i,k) == 0) 
      StructConst(NofRoots-i+1,i,k) = -kFct(l) * Tmp;
            
      if kFct(l) * Tmp - round( kFct(l) * Tmp)
        NofRoots
        Rank
        kFct(l)
        Tmp
        disp(['ROOTS(:,l)'' * metric * ROOTS(:,l)'])
        ROOTS(:,l)' * metric * ROOTS(:,l)
        disp(['ROOTS(:,i)'' * metric * ROOTS(:,i)'])
        disp(['ROOTS(:,i=' num2str(i) ')'])
        ROOTS(:,i)
        disp(['ROOTS(:,l=' num2str(l) ')'])
        ROOTS(:,l)
%        keyboard
      end
      
    end  
  end
  

% roots
%
%  [-N, -N+1, ... -2, -1, 1, 2, R, R+1, ... N-1, N]
%
%  [1, 2, R]                   :  simple roots
%  [-N, -N+1, ... -2, -1]      :  negative roots
%  [1, 2, R, R+1, ... N-1, N]  :  positive roots

%
%  consider
%
%    [H_{alpha}, E_{beta}] 
%
%  and
%
%    [E_{alpha}, H_{beta}]  
%

  for i = SimpleRootsIdx,
    for j = 1:NofRoots,

      Tmp = 2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
                (ROOTS(:,i)' * metric * ROOTS(:,i));

%       if j-NofRoots/2 >= 1 && j-NofRoots/2 <= Rank  
%         Sav(i-NofRoots/2,j-NofRoots/2) = Tmp;        
%       end
      
      if 0 & ~isempty( find( SimpleRootsIdx == j, 1))  
        cartan(i-NofRoots/2,j-NofRoots/2)
        cartan
        i,j
        Tmp
        keyboard
      end
              
%       Tmp = 2 * (ROOTS(:,j)' * metric * ROOTS(:,i)) / ...
%                 (ROOTS(:,j)' * metric * ROOTS(:,j));
%       Tmp = 2 * (ROOTSRS(:,i)' * metric * ROOTSRS(:,j)) / ...
%                 (ROOTSRS(:,i)' * metric * ROOTSRS(:,i));              
%       if Tmp ~= Tmp2,
%         Tmp
%         Tmp2
%         keyboard
%       end
      k = j;              
      mlassert( StructConst(i+NofPosRoots,j,k) == 0) 
      StructConst(i+NofPosRoots,j,k) = Tmp;      
% structure constants are antisymmetric in first two indices
      mlassert( StructConst(j,i+NofPosRoots,k) == 0) 
      StructConst(j,i+NofPosRoots,k) = -Tmp;      
    end
  end

%   Sav
%   cartan
%   keyboard
  
  for k = 1:N,
    % f^k_{i,j} -> (f_i)_{k,j}
    % swap 2 and 3 index
    disp( ['structure constants k = ' num2str( k) ' :'])
    disp( StructConst(:,:,k))
    disp(' ')
  end
  keyboard


%
%  consistency checks
%
  Tmp = abs( StructConst - round( StructConst));
  MaxDevFromInteger = max( Tmp(:));
  
  if MaxDevFromInteger > 1e-6
    for i = 1:N,
      Tmp = abs( StructConst(:,:,i) - round( StructConst(:,:,i)));  
      TmpMaxDevFromInteger = max( Tmp(:));
      if TmpMaxDevFromInteger > 1e-6
        disp( squeeze( StructConst(:,:,i)))
        keyboard
      end
    end
  end
  
  priv_assert( MaxDevFromInteger < 1e-6);
  
  Tmp = abs( StructConst + permute( StructConst, [2 1 3]));
  MaxDevFromInteger = max( Tmp(:));
  priv_assert( MaxDevFromInteger < 1e-6);

%   ROOTS'
%   keyboard
  

%   for i = SimpleRootsIdx,
%     for j = 1:NofRoots,
% %       i
% %       i+NofRoots/2
%       mlassert( all( StructConst(j,i+NofRoots/2,:) == 0))
%       StructConst(j,i+NofRoots/2,:) = -StructConst(i+NofRoots/2,j,:);      
%     end
%   end
%  keyboard
  
%  StructConst
  
%  
%   g_{i,j} = \sum_{k,l} c^k_{i,l} * c^l_{j,k}
%  
  
  N = NofRoots + Rank;

%   g = zeros( N, N);
% 
%   for i = 1:N,
%     for j = 1:N,
%       for k = 1:N,
%         for l = 1:N,
%           g(i,j) = g(i,j) + StructConst(i,l,k) * StructConst(j,k,l);
%         end
%       end
%     end
%   end
  
  
%   for i = 1:N,
%     % f^k_{i,j} -> (f_i)_{k,j}
%     % swap 2 and 3 index
%     disp( ['generator X_' num2str( i) ' :'])
%     disp( squeeze( StructConst(i,:,:))')
%     disp(' ')
%   end
  

%
%  now loop thru all N^2 (N : diemnsion of Lie algebra) possibilities and 
%  check that the commutators are what we expect.
%
  AdjRep = permute( StructConst, [3 2 1]); 
  
%   AdjRep
%   keyboard

  if 1,  
    for i = 1:N,
      disp( ['i = ' num2str(i) ' / ' num2str( N)])
      for j = 1:N,

  %
  %  commutator
  %
%         Tmp1 = squeeze( StructConst(i,:,:))' * squeeze( StructConst(j,:,:))' - ...
%                squeeze( StructConst(j,:,:))' * squeeze( StructConst(i,:,:))'; 
             
        Tmp1 = AdjRep(:,:,i) * AdjRep(:,:,j) - AdjRep(:,:,j) * AdjRep(:,:,i);

%        priv_assert( Tmp1 == Tmp3)
             
        Tmp2 = zeros( N, N);     
        NofSum = 0; 
        for k = 1:N,
          if StructConst(i,j,k)
            NofSum = NofSum + 1; 
          end
%          Tmp2 = Tmp2 + StructConst(i,j,k) * squeeze( StructConst(k,:,:))';
          Tmp2 = Tmp2 + StructConst(i,j,k) * AdjRep(:,:,k);
        end
        
        if ( NofSum > Rank)
          disp( ['i = ' num2str( i) ', j = ' num2str( j)])
          StructConst(i,j,:)
          keyboard
        end
        
        if any( any( Tmp1 ~= Tmp2))

          if 0,
            disp( [ upper( mfilename) ': expected  [X_' num2str(i) ', X_' ...
              num2str(j) '] = X_' num2str(k)])
            disp( [ upper( mfilename) ': X_' num2str(k) ' ='])
            if all( all( Tmp1 == 0))
              disp( 0)
            else
              disp( Tmp1)    
            end
            disp( [ upper( mfilename) ': but calculation from AdjRep yields [X_' num2str(i) ', X_' num2str(j) '] ='])
            if all( all( Tmp2 == 0))
              disp( 0)
            else
              disp( Tmp2)    
            end
  %           disp( [ upper( mfilename) ': StructConst(' ...
  %             num2str(i) ',' num2str(j) ',' num2str(k) ') = ' num2str( StructConst(i,i,k))])
          end
          
          [iSel,jSel] = find( Tmp1 ~= Tmp2);
          for k = 1:length( iSel),
            disp( [ 'i = ' num2str(i) ', j = ' num2str(j) ...
              ' -> Tmp(' num2str(iSel(k)) ',' num2str(jSel(k)) ') : ' ...
              num2str( Tmp1(iSel(k),jSel(k))) ' <-> ' num2str( Tmp2(iSel(k),jSel(k)))])
          end
          TestFailed = 1;
%          keyboard
        end

      end
    end

  end
  
  if ~TestFailed
    disp( [ upper( mfilename) ': ... test adjoint representation passed.'])
  else
    disp( [ upper( mfilename) ': *** test adjoint representation failed! ***'])
  end
  
  return
  
  keyboard

  
  
  
  
  
  
  
  
  
  
  
  
  
  
  for i = 1:N,
    for j = 1:N,
 
      if i == 1 && j == 4
        keyboard
      end
      
%
%  commutator
%
      Tmp = squeeze( StructConst(i,:,:))' * squeeze( StructConst(j,:,:))' - ...
            squeeze( StructConst(j,:,:))' * squeeze( StructConst(i,:,:))'; 

      if i <= NofRoots && j <= NofRoots

        if i + j == NofRoots+1
% [E_{alpha}, E_{-alpha}], alpha is simple root
% result of this commutator should be within Cartan subalgebra
          if ~isempty( find( SimpleRootsIdx == i, 1))
            k = i + NofPosRoots;
            local_comparegenerators( Tmp, ...
              squeeze( StructConst(k,:,:))', i, j, k);
          elseif ~isempty( find( SimpleRootsIdx == j, 1))
            k = j + NofPosRoots;
            local_comparegenerators( Tmp, ...
              -1 * squeeze( StructConst(k,:,:))', i, j, k);
          else  
% [E_{alpha}, E_{-alpha}] with alpha nonsimple root is 
% linear combination of cartan subalgebra elements  
            local_validateCartanGenerator( Tmp, StructConst(ZeroRootsIdx,:,:));

            [i, j]
            disp( Tmp)
            keyboard
          end
        else
% [E_{alpha}, E_{beta}] with beta ~= -alpha          
          k = SUMOFROOTS(i,j);
          if k > 0
            local_comparegenerators( Tmp, ...
              NCoeff(i,j) * squeeze( StructConst(k,:,:))', i, j, k); 
          elseif any( any( Tmp))
            disp( [ upper( mfilename) ': expected  [X_' num2str(i) ', X_' num2str(j) '] = 0'])
            disp( [ upper( mfilename) ': but found [X_' num2str(i) ', X_' num2str(j) '] ='])
            disp( Tmp)    
            keyboard
          end
        end
        
      elseif i > NofRoots && j > NofRoots && any( any( Tmp))
%
%  elements in Cartan subalgebra always commute 
%
        disp( [ upper( mfilename) ': expected  [X_' num2str(i) ', X_' num2str(j) '] = 0'])
        disp( [ upper( mfilename) ': but found [X_' num2str(i) ', X_' num2str(j) '] ='])
        disp( Tmp)    
        keyboard
      else  
        [i, j]  
%         continue
%         keyboard
      end
    end
  end
  
  
  keyboard
  return
  
  
  i = 1;
  j = 4;
    

  disp( [ ' [X_' num2str(i) ', X_' num2str(j) '] = '])
  disp( Tmp)

  k = SUMOFROOTS(i,j);
  if k > 0
    disp( [ ' [X_' num2str(i) ', X_' num2str(j) '] ~ X_' num2str(k) ])
    disp( ['generator X_' num2str( k) ' :'])
    disp( squeeze( StructConst(k,:,:))')
  else
    disp( [ ' [X_' num2str(i) ', X_' num2str(j) '] = 0'])
  end
  
  keyboard
  return
  
  
%  size( ROOTS, 1) : rank = number of simple roots
%  size( ROOTS, 2) : number of (pos. & neg.) roots
  
  nofRoots = size( ROOTS, 2);

  for i = 1:nofRoots,
    for j = 1:nofRoots,
      for k = 1:nofRoots,
  
        iPj = SUMOFROOTS( i, j);
        
        if iPj == 0,
          continue
        end
          
        if iPj ~= ROOTFLIPSIGN(k)
          continue
        end
           
        aa     = (ROOTS(:,i)' * metric * ROOTS(:,i));
        bb     = (ROOTS(:,j)' * metric * ROOTS(:,j));
        aPbaPb = (ROOTS(:,iPj)' * metric * ROOTS(:,iPj));
        
        if NCoeff(j,k) / NCoeff(k,i) == aa / bb && ...
           NCoeff(k,i) / NCoeff(i,j) == bb / aPbaPb
         continue
        end
        
        if NCoeff(j,k) == aa / bb * NCoeff(k,i) && ...
           NCoeff(k,i) == bb / aPbaPb * NCoeff(i,j)
         continue
        end

        NCoeff(j,k)
        NCoeff(k,i)
        NCoeff(i,j)
        
        (ROOTS(:,i)' * metric * ROOTS(:,i)) 
        (ROOTS(:,j)' * metric * ROOTS(:,j))
        (ROOTS(:,iPj)' * metric * ROOTS(:,iPj))
        
        disp( sprintf( 'i  = %2d, j  = %2d, k  = %2d', i, j, k))

%        i,j,k
        keyboard

      end
    end
  end

%  disp( [upper( mfilename) ': Theorem 8.1 fulfilled!'])
  
end

% -------------------------------------------------------------------------
%
%  compare equality of two generators
%
function local_comparegenerators( Tmp1, Tmp2, i, j, k)

  if ~any( any( Tmp1 ~= Tmp2))
    return
  end
 
  disp( [ upper( mfilename) ': expected  [X_' num2str(i) ', X_' ...
    num2str(j) '] = X_' num2str(k)])
  disp( [ upper( mfilename) ': X_' num2str(k) ' ='])
  disp( Tmp2)    
  disp( [ upper( mfilename) ': but found [X_' num2str(i) ', X_' num2str(j) '] ='])
  disp( Tmp1)    
  keyboard

end

function local_validateCartanGenerator( Tmp, ZeroGenerator)

  NofGen = size( ZeroGenerator, 1);
  for i = 1:NofGen
    disp( [ upper( mfilename) ': H_' num2str(i) ' ='])
    disp( squeeze( ZeroGenerator(i,:,:))')
  end
  Tmp
  keyboard
  
end
