function priv_validatejacobiidentity( metric, NCoeff)
%PRIV_VALIDATEJACOBIIDENTITY  check if Jacobi identity is fulfilled
%
%  priv_validatejacobiidentity( metric, NCoeff)
%

% (c) G. Beyerle 2012

  global ROOTS
  global SUMOFROOTSX
  global SUMOFROOTS
%   global SIGNOFROOTS
  global ROOTSORDER
  global PCOEFF

  nofRoots = size( ROOTS, 2);

  CartanX = local_initCARTANX( metric);

  NR = nofRoots;

  NCoeffX = ones(2*NR+1,2*NR+1)*(-999);
  NCoeffX(1:NR,1:NR) = NCoeff;

  NCoeffX(NR+(1:NR),NR+(1:NR)) = 0;

  NCoeffX(NR+(1:NR),1:NR) = CartanX;
  NCoeffX(1:NR,NR+(1:NR)) = -CartanX';

%     NCoeffX
%     keyboard

  disp( [upper( mfilename) ': checking Jacobi identity ...'])

  function jjStr = nested_tr(ii)
    if ii == 2*NR+1,
      jjStr = '0';
    elseif ii > NR && ii <= 2*NR,
      jjStr = ['z' num2str(ii)];
    else
      jjStr = num2str(ii);
    end
  end
  
  NofFailed = 0; 

  for i = 1:nofRoots,
    for j = 1:nofRoots,
      for k = 1:nofRoots,

%         if i == 1 && j == 2 && k == 10,
%           jk = 13
%           local_getcoeff2( jk, i, NCoeffX)
%           jk = jk - NR;
%           2 * ROOTS(:,jk)' * metric * ROOTS(:,i) / ...
%               (ROOTS(:,i)' * metric * ROOTS(:,i))
%           2 * ROOTS(:,jk)' * metric * ROOTS(:,i) / ...
%              (ROOTS(:,jk)' * metric * ROOTS(:,jk))
%           
%           keyboard
%         end

        ijCof = local_getcoeff2( i, j, NCoeff);
        jkCof = local_getcoeff2( j, k, NCoeff);
        kiCof = local_getcoeff2( k, i, NCoeff);

        ij     = SUMOFROOTSX(i,j);
        jk     = SUMOFROOTSX(j,k);
        ki     = SUMOFROOTSX(k,i);
        

%          ijkCof = NCoeffX(ij,k);
%          jkiCof = NCoeffX(jk,i);
%          kijCof = NCoeffX(ki,j);
        ijkCof = local_getcoeff2( ij, k, NCoeffX);
        jkiCof = local_getcoeff2( jk, i, NCoeffX);
        kijCof = local_getcoeff2( ki, j, NCoeffX);

        ijk    = SUMOFROOTSX(ij,k);
        jki    = SUMOFROOTSX(jk,i);
        kij    = SUMOFROOTSX(ki,j);

        if ( kij == 2*nofRoots+1 && ...
             ijk == 2*nofRoots+1 && ...
             jki == 2*nofRoots+1 )
% all three terms are zero             
          continue
        end

        if ( ~ijCof && ~jkCof && ~kiCof )
% all coefficients of 'inner' commutators terms are zero             
          continue
        end

        if ijCof*ijkCof == 0 && jkCof*jkiCof == -kiCof*kijCof && jki == kij
          continue
        end
        if jkCof*jkiCof == 0 && kiCof*kijCof == -ijCof*ijkCof && kij == ijk
          continue
        end
        if kiCof*kijCof == 0 && ijCof*ijkCof == -jkCof*jkiCof && ijk == jki
          continue
        end
        if (ijk <= nofRoots) && ijk == jki && jki == kij && ...
            ijCof*ijkCof + jkCof*jkiCof + kiCof*kijCof == 0
          continue
        end
          
        
        if ( ijk >= nofRoots+1 && ijk <= 2*nofRoots && ...
           jki >= nofRoots+1 && jki <= 2*nofRoots && ...
           kij >= nofRoots+1 && kij <= 2*nofRoots )
          continue
        end

        disp( '-----------')
        disp( sprintf( 'i  = %2d, j  = %2d, k  = %2d', i, j, k))
        disp( sprintf( 'ij = %2d, jk = %2d, ki = %2d', ij, jk, ki))
        disp( sprintf( 'ijk= %2d, jki= %2d, kij= %2d', ijk, jki, kij))
        disp(' ')
        disp( sprintf( '[[%d,%d],%d] + [[%d,%d],%d] + [[%d,%d],%d] = ', ...
          i,j,k, j,k,i, k,i,j))

        disp( sprintf( '(%d)*[%s,%d] + (%d)*[%s,%d] + (%d)*[%s,%d] = ', ...
          ijCof,nested_tr(ij),k, jkCof,nested_tr(jk),i, kiCof,nested_tr(ki),j))

        disp( sprintf( '(%d)*(%d)*[%s] + (%d)*(%d)*[%s] + (%d)*(%d)*[%s] = ', ...
          ijCof,ijkCof,nested_tr(ijk), jkCof,jkiCof,nested_tr(jki), kiCof,kijCof,nested_tr(kij)))

        disp( sprintf( '(%d)*[%s] + (%d)*[%s] + (%d)*[%s]', ...
          ijCof*ijkCof,nested_tr(ijk), jkCof*jkiCof,nested_tr(jki), kiCof*kijCof,nested_tr(kij)))

        NCoeff
        SUMOFROOTS
%        ROOTSORDER
%        SUMOFROOTSX
%        PCOEFF
        ROOTSORDER
        ROOTS
        keyboard

        NofFailed = NofFailed+1; 
      end
    end
  end
  
%  NofFailed 
  disp( [upper( mfilename) ': ... Jacobi identity valid'])
%  keyboard
  
end

% -------------------------------------------------------------------------
%
%
%
%
%
function CartanX = local_initCARTANX( metric)

  global ROOTS
              
  numrts = size( ROOTS, 2);

  CartanX = zeros( numrts, numrts);
  
  for i = 1:numrts,
    for j = 1:numrts,
      
      CartanX(i,j) =  2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
                          (ROOTS(:,i)' * metric * ROOTS(:,i));
%      CartanX(i,j) =  2 * (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
%                          (ROOTS(:,j)' * metric * ROOTS(:,j));

%       CartanX(i,j) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
%                         (ROOTS(:,i)' * metric * ROOTS(:,i));
%       CartanX(i,j) =      (ROOTS(:,i)' * metric * ROOTS(:,j)) / ...
%                           (ROOTS(:,i)' * metric * ROOTS(:,i));
%       CartanX(j,i) = 2 * ROOTS(:,i)' * metric * ROOTS(:,j) / ...
%                         (ROOTS(:,i)' * metric * ROOTS(:,i));
    end
  end

end

% -------------------------------------------------------------------------
%
%
%
%
function Cof = local_getcoeff2( i, j, NCoeff)
        
  global ROOTFLIPSIGN
  global SIGNOFROOTS

  if ( i == ROOTFLIPSIGN( j))
% pos/neg root pair!    
    Cof  = 1 * SIGNOFROOTS(i);
%    Cof  = 1;
  else
    Cof  = NCoeff(i,j);
  end
  
end
  
